Package: rredlist
Type: Package
Title: 'IUCN' Red List Client
Description: 'IUCN' Red List (<https://api.iucnredlist.org/>) client.
    The 'IUCN' Red List is a global list of threatened and endangered species.
    Functions cover all of the Red List 'API' routes. An 'API' key is required.
Version: 1.1.1
Authors@R: 
    c(person(given = "William",
             family = "Gearty",
             role = c("aut", "cre"),
             email = "willgearty@gmail.com",
             comment = c(ORCID = "0000-0003-0076-3262")),
      person(given = "Scott",
             family = "Chamberlain",
             role = c("aut"),
             email = "myrmecocystus@gmail.com",
             comment = c(ORCID = "0000-0003-1444-9135")),
      person(given = "rOpenSci",
             role = "fnd",
             comment = c(ROR = "019jywm96")),
      person(given = "Maëlle",
             family = "Salmon",
             role = "ctb",
             email = "maelle.salmon@yahoo.se",
             comment = c(ORCID = "0000-0002-2815-0399")),
      person(given = "Kevin",
             family = "Cazelles",
             role = "rev",
             email = "kevin.cazelles@insileco.io", 
             comment = c(ORCID = "0000-0001-6619-9874")),
      person(given = "Stephanie",
             family = "Hazlitt",
             role = "rev",
             email = "stephhazlitt@gmail.com",
             comment = c(ORCID = "0000-0002-3161-2304"))
      )
License: MIT + file LICENSE
URL: https://docs.ropensci.org/rredlist/,
        https://github.com/ropensci/rredlist
BugReports: https://github.com/ropensci/rredlist/issues
Encoding: UTF-8
Language: en-US
Imports: crul (>= 0.3.8), jsonlite (>= 1.1), lifecycle, cli, curl,
        rlang
Suggests: spelling, testthat, vcr (>= 0.6.0), knitr, rmarkdown,
        webmockr, microbenchmark, ggplot2, dplyr, tibble, tidyr,
        patchwork, vdiffr
RoxygenNote: 7.3.2.9000
VignetteBuilder: knitr
X-schema.org-applicationCategory: Biodiversity
X-schema.org-keywords: IUCN, biodiversity, API, web-services, traits,
        habitat, species, conservation
X-schema.org-isPartOf: https://ropensci.org
NeedsCompilation: no
Packaged: 2025-09-03 19:20:32 UTC; willg
Author: William Gearty [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0076-3262>),
  Scott Chamberlain [aut] (ORCID:
    <https://orcid.org/0000-0003-1444-9135>),
  rOpenSci [fnd] (ROR: <https://ror.org/019jywm96>),
  Maëlle Salmon [ctb] (ORCID: <https://orcid.org/0000-0002-2815-0399>),
  Kevin Cazelles [rev] (ORCID: <https://orcid.org/0000-0001-6619-9874>),
  Stephanie Hazlitt [rev] (ORCID:
    <https://orcid.org/0000-0002-3161-2304>)
Maintainer: William Gearty <willgearty@gmail.com>
Repository: CRAN
Date/Publication: 2025-09-04 07:20:02 UTC

# Generated by roxygen2: do not edit by hand

export(rl_actions)
export(rl_actions_)
export(rl_api_version)
export(rl_assessment)
export(rl_assessment_)
export(rl_assessment_extract)
export(rl_assessment_list)
export(rl_categories)
export(rl_categories_)
export(rl_citation)
export(rl_class)
export(rl_class_)
export(rl_common_names)
export(rl_common_names_)
export(rl_comp_groups)
export(rl_comp_groups_)
export(rl_countries)
export(rl_countries_)
export(rl_extinct)
export(rl_extinct_)
export(rl_extinct_wild)
export(rl_extinct_wild_)
export(rl_family)
export(rl_family_)
export(rl_faos)
export(rl_faos_)
export(rl_green)
export(rl_green_)
export(rl_growth_forms)
export(rl_growth_forms_)
export(rl_habitats)
export(rl_habitats_)
export(rl_history)
export(rl_history_)
export(rl_kingdom)
export(rl_kingdom_)
export(rl_measures)
export(rl_measures_)
export(rl_narrative)
export(rl_narrative_)
export(rl_occ_country)
export(rl_occ_country_)
export(rl_order)
export(rl_order_)
export(rl_phylum)
export(rl_phylum_)
export(rl_pop_trends)
export(rl_pop_trends_)
export(rl_realms)
export(rl_realms_)
export(rl_regions)
export(rl_regions_)
export(rl_research)
export(rl_research_)
export(rl_scopes)
export(rl_scopes_)
export(rl_search)
export(rl_search_)
export(rl_sis)
export(rl_sis_)
export(rl_sis_latest)
export(rl_sp)
export(rl_sp_)
export(rl_sp_category)
export(rl_sp_category_)
export(rl_sp_citation)
export(rl_sp_citation_)
export(rl_sp_count)
export(rl_sp_count_)
export(rl_sp_country)
export(rl_sp_country_)
export(rl_species)
export(rl_species_)
export(rl_species_latest)
export(rl_stresses)
export(rl_stresses_)
export(rl_synonyms)
export(rl_synonyms_)
export(rl_systems)
export(rl_systems_)
export(rl_threats)
export(rl_threats_)
export(rl_use_and_trade)
export(rl_use_and_trade_)
export(rl_use_iucn)
export(rl_version)
export(scale_color_iucn)
export(scale_colour_iucn)
export(scale_discrete_iucn)
export(scale_fill_iucn)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_alert_warning)
importFrom(cli,cli_code)
importFrom(cli,cli_end)
importFrom(cli,cli_h2)
importFrom(cli,cli_li)
importFrom(cli,cli_ol)
importFrom(cli,cli_progress_bar)
importFrom(cli,cli_progress_done)
importFrom(cli,cli_progress_update)
importFrom(crul,HttpClient)
importFrom(curl,nslookup)
importFrom(jsonlite,fromJSON)
importFrom(lifecycle,deprecated)
importFrom(rlang,check_installed)
importFrom(rlang,expr)
importFrom(stats,setNames)
importFrom(utils,bibentry)
importFrom(utils,packageVersion)

# rredlist <img src="man/figures/logo.png" align="right" width="120" />



[![Status at rOpenSci Software Peer Review](https://badges.ropensci.org/663_status.svg)](https://github.com/ropensci/software-review/issues/663)
[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![cran version](https://www.r-pkg.org/badges/version/rredlist)](https://cran.r-project.org/package=rredlist)
[![R-check](https://github.com/ropensci/rredlist/actions/workflows/R-check.yml/badge.svg)](https://github.com/ropensci/rredlist/actions/workflows/R-check.yml)
[![codecov.io](https://codecov.io/gh/ropensci/rredlist/coverage.svg)](https://app.codecov.io/gh/ropensci/rredlist)
[![rstudio mirror downloads](https://cranlogs.r-pkg.org/badges/rredlist)](https://github.com/r-hub/cranlogs.app)

`rredlist` is an R client for the IUCN Red List API (https://api.iucnredlist.org). The [IUCN Red List](https://www.iucnredlist.org/) is the world’s most comprehensive information source on the global extinction risk status of animal, fungus, and plant species. This package provides access via R to the various data contained within this database which span range details, population size, habitat and ecology, use and/or trade, threats, and conservation actions. The functions within the package cover all endpoints of the IUCN Red List web API, which are documented [here](https://api.iucnredlist.org/api-docs/index.html).

## Installation

CRAN


``` r
install.packages("rredlist")
```

Development version


``` r
remotes::install_github("ropensci/rredlist")
# OR
install.packages("rredlist", repos = "https://ropensci.r-universe.dev/")
```

## Authentication

Use of this package requires an IUCN API key which can be acquired at https://api.iucnredlist.org/users/sign_up. There is a helper function to help you get the key and store it properly:


``` r
rredlist::rl_use_iucn()
```

**Keep this key private.** You can pass the key in to each function via the key parameter, but it’s better to store the key either as an environment variable (`IUCN_REDLIST_KEY`) or an R option (`iucn_redlist_key`) - we recommend using the former option. Note that there is not a default API key that is used as a fallback, and the package will not function without providing/storing your own API key.

## Example usage

### Loading the package

``` r
library("rredlist")
```

### Search for assessments for a particular species

``` r
rl_species("Gorilla", "gorilla")$assessments
#>    year_published latest possibly_extinct possibly_extinct_in_the_wild sis_taxon_id
#> 1            2016  FALSE            FALSE                        FALSE         9404
#> 2            2018   TRUE            FALSE                        FALSE         9404
#> 3            2016  FALSE            FALSE                        FALSE         9404
#> 4            2008  FALSE            FALSE                        FALSE         9404
#> 5            2007  FALSE            FALSE                        FALSE         9404
#> 6            2000  FALSE            FALSE                        FALSE         9404
#> 7            1996  FALSE            FALSE                        FALSE         9404
#> 8            1994  FALSE            FALSE                        FALSE         9404
#> 9            1990  FALSE            FALSE                        FALSE         9404
...
```

### Search for assessments that recommend particular conservation actions

#### Get a list of all conservation actions

``` r
rl_actions()
#> $conservation_actions
#>                                              en  code
#> 1                         Land/water protection     1
#> 2                          Site/area protection   1_1
#> 3                 Resource & habitat protection   1_2
#> 4                         Land/water management     2
#> 5                          Site/area management   2_1
#> 6          Invasive/problematic species control   2_2
#> 7         Habitat & natural process restoration   2_3
#> 8                            Species management     3
...
```

#### Return assessments with a particular conservation action

``` r
rl_actions("2_2", all = FALSE)$assessments
#>    year_published latest possibly_extinct possibly_extinct_in_the_wild sis_taxon_id
#> 1            2019   TRUE            FALSE                        FALSE    132523146
#> 2            2019   TRUE            FALSE                        FALSE        10767
#> 3            2013   TRUE            FALSE                        FALSE         1078
#> 4            2019   TRUE            FALSE                        FALSE    132521900
#> 5            2020  FALSE            FALSE                        FALSE         1086
#> 6            2019   TRUE            FALSE                        FALSE         1117
#> 7            2019  FALSE            FALSE                        FALSE        11797
#> 8            2021   TRUE            FALSE                        FALSE        12124
#> 9            2025   TRUE            FALSE                        FALSE    232775771
...
```

## Logo

<img src="man/figures/logo.png" align="right" width="120" />

The `rredlist` logo showcases a silhouette of a [Javan rhinoceros](https://www.iucnredlist.org/species/19495/18493900) (_Rhinoceros sondaicus_), one of the most endangered mammal species on the planet. The species has suffered extreme population decline due to habitat loss and poaching, with only ~75 individuals alive in the wild today, all in Ujung Kulon National Park, a [UNESCO World Heritage Site](https://whc.unesco.org/en/list/608) in Java, Indonesia. Despite recent conservation efforts, [poaching continues](https://www.savetherhino.org/asia/indonesia/poaching-gangs-claim-to-have-killed-one-third-of-the-remaining-javan-rhino-population/); further, the small population is extremely susceptible to inbreeding, disease, and further habitat loss due to the rampant spreading of local palm trees. You can read more about the Javan rhino on the [IUCN Red List](https://www.iucnredlist.org/species/19495/18493900), [World Wildlife Fund](https://www.worldwildlife.org/species/javan-rhino), and [International Rhino Foundation](https://rhinos.org/about-rhinos/rhino-species/javan-rhino/).

This work, "rredlist logo", is adapted from ["Javan rhino silhouette"](https://creazilla.com/media/silhouette/64313/javan-rhino) by [Creazilla](https://creazilla.com/), used under [CC BY 4.0](https://creativecommons.org/licenses/by/4.0/). "rredlist logo" is licensed under [CC BY 4.0](https://creativecommons.org/licenses/by/4.0/) by William Gearty.

## Meta

* Please [report any issues or bugs](https://github.com/ropensci/rredlist/issues).
* License: MIT
* Get citation information for `rredlist` in R doing `citation(package = 'rredlist')`
* Please note that this package is released with a [Contributor Code of Conduct](https://ropensci.org/code-of-conduct/). By contributing to this project, you agree to abide by its terms.

[![rofooter](https://ropensci.org/public_images/github_footer.png)](https://ropensci.org)

## Contributors


<!-- ALL-CONTRIBUTORS-LIST:START - Do not remove or modify this section -->
<!-- prettier-ignore-start -->
<!-- markdownlint-disable -->

All contributions to this project are gratefully acknowledged using the [`allcontributors` package](https://github.com/ropensci/allcontributors) following the [allcontributors](https://github.com/all-contributors/all-contributors) specification. Contributions of any kind are welcome!

### Code

<table>

<tr>
<td align="center">
<a href="https://github.com/sckott">
<img src="https://avatars.githubusercontent.com/u/577668?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/commits?author=sckott">sckott</a>
</td>
<td align="center">
<a href="https://github.com/willgearty">
<img src="https://avatars.githubusercontent.com/u/7232514?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/commits?author=willgearty">willgearty</a>
</td>
<td align="center">
<a href="https://github.com/maelle">
<img src="https://avatars.githubusercontent.com/u/8360597?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/commits?author=maelle">maelle</a>
</td>
<td align="center">
<a href="https://github.com/jeffreyhanson">
<img src="https://avatars.githubusercontent.com/u/3610005?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/commits?author=jeffreyhanson">jeffreyhanson</a>
</td>
<td align="center">
<a href="https://github.com/jeroen">
<img src="https://avatars.githubusercontent.com/u/216319?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/commits?author=jeroen">jeroen</a>
</td>
<td align="center">
<a href="https://github.com/KevCaz">
<img src="https://avatars.githubusercontent.com/u/1583534?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/commits?author=KevCaz">KevCaz</a>
</td>
</tr>

</table>


### Issue Authors

<table>

<tr>
<td align="center">
<a href="https://github.com/stevenpbachman">
<img src="https://avatars.githubusercontent.com/u/8523782?u=b2901ffea809dea3ea9abb6d5456c13a1ebeed0b&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Astevenpbachman">stevenpbachman</a>
</td>
<td align="center">
<a href="https://github.com/FVFaleiro">
<img src="https://avatars.githubusercontent.com/u/5278091?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3AFVFaleiro">FVFaleiro</a>
</td>
<td align="center">
<a href="https://github.com/git-og">
<img src="https://avatars.githubusercontent.com/u/37261897?u=1cdc45771453cd662653c0d5fd55731c9416b51d&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Agit-og">git-og</a>
</td>
<td align="center">
<a href="https://github.com/arw36">
<img src="https://avatars.githubusercontent.com/u/16709317?u=0f83d16890d2ec07ef1d9bab7f9d50d610b5de2a&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Aarw36">arw36</a>
</td>
<td align="center">
<a href="https://github.com/electricquad">
<img src="https://avatars.githubusercontent.com/u/47792441?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Aelectricquad">electricquad</a>
</td>
<td align="center">
<a href="https://github.com/bgerstner90">
<img src="https://avatars.githubusercontent.com/u/34172115?u=4f8a9fad0ab5e1104a4f6683a358cb3315ff1eeb&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Abgerstner90">bgerstner90</a>
</td>
<td align="center">
<a href="https://github.com/lime-n">
<img src="https://avatars.githubusercontent.com/u/68914515?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Alime-n">lime-n</a>
</td>
</tr>


<tr>
<td align="center">
<a href="https://github.com/marcelxelo">
<img src="https://avatars.githubusercontent.com/u/83825031?u=51ce8fb4dd74e65ebebb84e7021bf0beb1b8b66e&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Amarcelxelo">marcelxelo</a>
</td>
<td align="center">
<a href="https://github.com/cpavloud">
<img src="https://avatars.githubusercontent.com/u/32954293?u=c4b1ed82c4cec63ffcc448a9691991723010d8ef&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Acpavloud">cpavloud</a>
</td>
<td align="center">
<a href="https://github.com/Gopal-Murali">
<img src="https://avatars.githubusercontent.com/u/16218632?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3AGopal-Murali">Gopal-Murali</a>
</td>
<td align="center">
<a href="https://github.com/DanielIAvila">
<img src="https://avatars.githubusercontent.com/u/26021913?u=537bdb63e3f728e112bb3e5ce4fb5c45e8518eea&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3ADanielIAvila">DanielIAvila</a>
</td>
<td align="center">
<a href="https://github.com/michitobler">
<img src="https://avatars.githubusercontent.com/u/28713008?u=50786272806f2a4a98576401a00d346d17e12e4f&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Amichitobler">michitobler</a>
</td>
<td align="center">
<a href="https://github.com/martijnvandepol">
<img src="https://avatars.githubusercontent.com/u/108213692?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Amartijnvandepol">martijnvandepol</a>
</td>
<td align="center">
<a href="https://github.com/XuWeiEvo">
<img src="https://avatars.githubusercontent.com/u/126555867?u=a920a4434b0ffd89f4a7049615cc0bb98aa830d7&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3AXuWeiEvo">XuWeiEvo</a>
</td>
</tr>


<tr>
<td align="center">
<a href="https://github.com/makosiaaa">
<img src="https://avatars.githubusercontent.com/u/168120118?u=6d6c33c848362a33ec9539450068746c0587a9b3&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Amakosiaaa">makosiaaa</a>
</td>
<td align="center">
<a href="https://github.com/wk-ai">
<img src="https://avatars.githubusercontent.com/u/72534080?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Awk-ai">wk-ai</a>
</td>
<td align="center">
<a href="https://github.com/Lisa-Tedeschi">
<img src="https://avatars.githubusercontent.com/u/126464153?u=e305afad7c7938801d78a8e8da08633cbf171100&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3ALisa-Tedeschi">Lisa-Tedeschi</a>
</td>
<td align="center">
<a href="https://github.com/oldenfish">
<img src="https://avatars.githubusercontent.com/u/46573596?u=00ec35751b436213ad7a0bd51fc0d7c343443b3a&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Aoldenfish">oldenfish</a>
</td>
<td align="center">
<a href="https://github.com/laetitiatremblay">
<img src="https://avatars.githubusercontent.com/u/195579379?u=6d995e1d76cb44425ede7fd60973fb81c12c8e7f&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+author%3Alaetitiatremblay">laetitiatremblay</a>
</td>
</tr>

</table>


### Issue Contributors

<table>

<tr>
<td align="center">
<a href="https://github.com/olliewearn">
<img src="https://avatars.githubusercontent.com/u/22527233?u=1693575e4d6eccb775e4aca208983bdbc254abf5&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Aolliewearn">olliewearn</a>
</td>
<td align="center">
<a href="https://github.com/matthewlewis896">
<img src="https://avatars.githubusercontent.com/u/54063056?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Amatthewlewis896">matthewlewis896</a>
</td>
<td align="center">
<a href="https://github.com/mairindeith">
<img src="https://avatars.githubusercontent.com/u/7770415?u=a3d879dfc6ac65913dab219b05ea68ae69e9b176&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Amairindeith">mairindeith</a>
</td>
<td align="center">
<a href="https://github.com/MirzaCengic">
<img src="https://avatars.githubusercontent.com/u/19353225?u=9f14ce041845ab3395bbc80f51876f06ba375e71&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3AMirzaCengic">MirzaCengic</a>
</td>
<td align="center">
<a href="https://github.com/rdornas">
<img src="https://avatars.githubusercontent.com/u/25906665?u=092efd13ddf08cef90abfe29ea0293cf24a82fde&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Ardornas">rdornas</a>
</td>
<td align="center">
<a href="https://github.com/awpark">
<img src="https://avatars.githubusercontent.com/u/6181375?u=9839305f921edd2dd99e97a2d9ffbf5d107b4936&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Aawpark">awpark</a>
</td>
<td align="center">
<a href="https://github.com/ingomiller">
<img src="https://avatars.githubusercontent.com/u/74630511?u=41ed92c36c79dab4a3a77aaca555b66c733eb893&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Aingomiller">ingomiller</a>
</td>
</tr>


<tr>
<td align="center">
<a href="https://github.com/yabellini">
<img src="https://avatars.githubusercontent.com/u/2473676?u=2ada2d3ef6403fabc25fa323110791a20031ab85&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Ayabellini">yabellini</a>
</td>
<td align="center">
<a href="https://github.com/benscarlson">
<img src="https://avatars.githubusercontent.com/u/566703?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Abenscarlson">benscarlson</a>
</td>
<td align="center">
<a href="https://github.com/mcauchoix">
<img src="https://avatars.githubusercontent.com/u/727164?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Amcauchoix">mcauchoix</a>
</td>
<td align="center">
<a href="https://github.com/magpiedin">
<img src="https://avatars.githubusercontent.com/u/8563362?u=b72e5ec9689d498dd282717ce70c8b0dd22f5ee8&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Amagpiedin">magpiedin</a>
</td>
<td align="center">
<a href="https://github.com/klausriede">
<img src="https://avatars.githubusercontent.com/u/12816652?u=d3458cfa913d3c322cddcdcf3f9d5d8b61349d7c&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Aklausriede">klausriede</a>
</td>
<td align="center">
<a href="https://github.com/simon-tarr">
<img src="https://avatars.githubusercontent.com/u/27423094?u=a720ad78ed9b2b721cf7c2ee7726b41465bf3c3d&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Asimon-tarr">simon-tarr</a>
</td>
<td align="center">
<a href="https://github.com/ToleranceTrading">
<img src="https://avatars.githubusercontent.com/u/45725570?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3AToleranceTrading">ToleranceTrading</a>
</td>
</tr>


<tr>
<td align="center">
<a href="https://github.com/omerome83">
<img src="https://avatars.githubusercontent.com/u/46061735?u=8662a93b7adba45420a2b44ff206f55b2e9a2b93&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Aomerome83">omerome83</a>
</td>
<td align="center">
<a href="https://github.com/Crastoman">
<img src="https://avatars.githubusercontent.com/u/92993185?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3ACrastoman">Crastoman</a>
</td>
<td align="center">
<a href="https://github.com/karolazvdo">
<img src="https://avatars.githubusercontent.com/u/101413438?u=66e4beb0a88c6c3aef073da3c59995311447453d&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Akarolazvdo">karolazvdo</a>
</td>
<td align="center">
<a href="https://github.com/Andresmrv3">
<img src="https://avatars.githubusercontent.com/u/136818946?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3AAndresmrv3">Andresmrv3</a>
</td>
<td align="center">
<a href="https://github.com/jzhongwen">
<img src="https://avatars.githubusercontent.com/u/143148014?u=8329a4bad9b98ad78624a877493c286684148c11&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Ajzhongwen">jzhongwen</a>
</td>
<td align="center">
<a href="https://github.com/malmagrok">
<img src="https://avatars.githubusercontent.com/u/153113378?u=f02bc2b5dff277aae35bf8f7d39c3dd813c46c84&v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Amalmagrok">malmagrok</a>
</td>
<td align="center">
<a href="https://github.com/nicholasZ16">
<img src="https://avatars.githubusercontent.com/u/180613393?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3AnicholasZ16">nicholasZ16</a>
</td>
</tr>


<tr>
<td align="center">
<a href="https://github.com/dcuadrac">
<img src="https://avatars.githubusercontent.com/u/182159965?v=4" width="100px;" alt=""/>
</a><br>
<a href="https://github.com/ropensci/rredlist/issues?q=is%3Aissue+commenter%3Adcuadrac">dcuadrac</a>
</td>
</tr>

</table>

<!-- markdownlint-enable -->
<!-- prettier-ignore-end -->
<!-- ALL-CONTRIBUTORS-LIST:END -->

#' Get the Red List API version
#'
#' Returns the current version number of the IUCN Red List of Threatened Species
#' API.
#'
#' @export
#' @param key (character) An IUCN API token. See [rl_use_iucn()].
#' @param ... Curl options passed to [HttpClient][crul::HttpClient()].
#' @return API version as character string.
#' @family stats
#' @examples \dontrun{
#' rl_api_version()
#' }
rl_api_version <- function(key = NULL, ...) {
  assert_is(key, "character")
  rl_parse(rr_GET("information/api_version", key, ...), TRUE)$api_version
}

#' Get the Red List version
#'
#' Returns the current version number of the IUCN Red List of Threatened Species
#'
#' @export
#' @param key (character) An IUCN API token. See [rl_use_iucn()].
#' @param ... Curl options passed to [HttpClient][crul::HttpClient()].
#' @return Red List version as character string.
#' @family stats
#' @examples \dontrun{
#' rl_version()
#' }
rl_version <- function(key = NULL, ...) {
  assert_is(key, "character")
  rl_parse(rr_GET("information/red_list_version", key, ...),
           TRUE)$red_list_version
}

context("rl_version functions")

test_that("rl_version works", {
  skip_on_cran()

  vcr::use_cassette("rl_version", {
    aa <- rl_version()
  })
  expect_is(aa, "character")
  expect_match(aa, "[0-9]{4}-[0-9]")
})

test_that("rl_api_version works", {
  skip_on_cran()

  vcr::use_cassette("rl_api_version", {
    aa <- rl_api_version()
  })
  expect_is(aa, "character")
  expect_match(aa, "v[0-9]")
})

test_that("rl_version curl options work", {
  skip_on_cran()

  expect_error(rl_version(timeout_ms = 1), "Timeout was reached")
})

#' SIS ID assessment summary
#'
#' Get an assessment summary for a particular taxonomic entity based on its ID
#' number from the [IUCN Species Information Service
#' (SIS)](https://www.iucnredlist.org/assessment/sis).
#'
#' @export
#' @param id (integer) The SIS ID of the taxonomic entity to look up.
#' @template all
#' @template curl
#' @template info
#' @family taxa
#' @examples \dontrun{
#' # Get assessment summary for species
#' ex1 <- rl_sis(id = 9404)
#' nrow(ex1$assessments)
#' }
rl_sis <- function(id, key = NULL, parse = TRUE, ...) {
  assert_is(parse, "logical")

  rl_parse(rl_sis_(id, key, ...), parse)
}

#' @export
#' @rdname rl_sis
rl_sis_ <- function(id, key = NULL, ...) {
  assert_is(key, "character")
  assert_is(id, c("integer", "numeric"))
  assert_n(id, 1)

  rr_GET(paste("taxa/sis", id, sep = "/"), key, ...)
}

#' SIS ID latest assessment
#'
#' Get the latest assessment for a particular taxonomic entity based on its ID
#' number from the [IUCN Species Information Service
#' (SIS)](https://www.iucnredlist.org/assessment/sis). Wraps [rl_sis()] and
#' [rl_assessment()].
#'
#' @export
#' @param id (integer) The SIS ID of the taxonomic entity to look up.
#' @param scope (character) An optional scope (see [rl_scopes()]) to filter the
#'   results by. The default, "1", returns the latest global assessment. If no
#'   assessments exist for the specified `scope`, the latest assessment across
#'   all scopes is returned. If `NULL`, the latest assessment across all scopes
#'   is returned.
#' @template all
#' @template curl
#' @template info
#' @family taxa
#' @examples \dontrun{
#' # Get latest assessment for species
#' ex1 <- rl_sis_latest(id = 9404)
#' ex1$stresses
#' }
rl_sis_latest <- function(id, scope = "1", key = NULL, parse = TRUE, ...) {
  assert_is(parse, "logical")

  tmp <- rl_sis(id, key, ...)$assessments
  if (!is.null(scope)) {
    ind <- vapply(tmp$scopes, function(x) scope %in% x$code, logical(1))
    if (any(ind)) {
      tmp <- tmp[ind, , drop = FALSE]
    } else {
      cli_alert_warning(paste(
        "No assessment(s) found for the specified scope.",
        "Returning the latest assessment across all scopes.")
      )
    }
  }
  if (any(tmp$latest, na.rm = TRUE)) {
    tmp <- subset(tmp, tmp$latest)
  }
  tmp$year_published <- as.numeric(as.character(tmp$year_published))
  ord <- order(tmp$year_published, decreasing = TRUE)
  tmp <- tmp[ord, , drop = FALSE]
  rl_assessment(id = tmp$assessment_id[1], key = key, parse = parse, ...)
}

#' Species assessment summary
#'
#' Get an assessment summary for a particular species (i.e., Latin binomial) or
#' subspecies/variety/subpopulation (i.e., Latin trinomial).
#'
#' @export
#' @param genus (character) The genus name of the species to look up.
#' @param species (character) The species epithet of the species to look up.
#' @param infra (character) An optional name of the subspecies or variety to
#'   look up.
#' @param subpopulation (character) An optional name of the geographically
#'   separate subpopulation to look up.
#' @details Geographically separate subpopulations of a species are defined as
#'   those populations that are so isolated from others of the same species that
#'   it is considered extremely unlikely that there is any genetic interchange.
#'   In general, listings of such subpopulations are restricted to those that
#'   have been isolated for a long period of time.
#'
#'   Assessments of subspecies, varieties, and geographically separate
#'   subpopulations must adhere to the same standards as for species
#'   assessments. However, these assessments are only included provided there is
#'   a global assessment of the species as a whole.
#'
#'   Infraspecific ranks such as formas, subvarieties, cultivars, etc are not
#'   included in the Red List.
#' @template all
#' @template curl
#' @template info
#' @family taxa
#' @examples \dontrun{
#' # Get assessment summary for species
#' ex1 <- rl_species(genus = "Fratercula", species = "arctica")
#' nrow(ex1$assessments)
#'
#' # Get assessment summary for subspecies
#' ex2 <- rl_species(genus = "Gorilla", species = "gorilla",
#'                   infra = "gorilla")
#' nrow(ex2$assessments)
#' }
rl_species <- function(genus, species, infra = NULL, subpopulation = NULL,
                       key = NULL, parse = TRUE, ...) {
  assert_is(parse, "logical")

  rl_parse(rl_species_(genus, species, infra, subpopulation, key, ...), parse)
}

#' @export
#' @rdname rl_species
rl_species_ <- function(genus, species, infra = NULL, subpopulation = NULL,
                        key = NULL, ...) {
  assert_is(key, "character")
  assert_is(genus, "character")
  assert_is(species, "character")
  assert_n(species, 1)
  assert_is(infra, "character")
  assert_is(subpopulation, "character")

  rr_GET("taxa/scientific_name", key,
         query = list(genus_name = genus, species_name = species,
                      infra_name = infra, subpopulation_name = subpopulation),
         ...)
}

#' Species latest assessment
#'
#' Get the latest assessment for a particular species (i.e., Latin binomial) or
#' subspecies/variety/subpopulation (i.e., Latin trinomial). Wraps
#' [rl_species()] and [rl_assessment()].
#'
#' @export
#' @param genus (character) The genus name of the species to look up.
#' @param species (character) The species epithet of the species to look up.
#' @param infra (character) An optional name of the subspecies or variety to
#'   look up.
#' @param subpopulation (character) An optional name of the geographically
#'   separate subpopulation to look up.
#' @param scope (character) An optional scope (see [rl_scopes()]) to filter the
#'   results by. The default, "1", returns the latest global assessment. If no
#'   assessments exist for the specified `scope`, the latest assessment across
#'   all scopes is returned. If `NULL`, the latest assessment across all scopes
#'   is returned.
#' @details Geographically separate subpopulations of a species are defined as
#'   those populations that are so isolated from others of the same species that
#'   it is considered extremely unlikely that there is any genetic interchange.
#'   In general, listings of such subpopulations are restricted to those that
#'   have been isolated for a long period of time.
#'
#'   Assessments of subspecies, varieties, and geographically separate
#'   subpopulations must adhere to the same standards as for species
#'   assessments. However, these assessments are only included provided there is
#'   a global assessment of the species as a whole.
#'
#'   Infraspecific ranks such as formas, subvarieties, cultivars, etc are not
#'   included in the Red List.
#' @template all
#' @template curl
#' @template info
#' @family taxa
#' @importFrom cli cli_alert_warning
#' @examples \dontrun{
#' # Get latest assessment for species
#' ex1 <- rl_species_latest(genus = "Fratercula", species = "arctica")
#' ex1$stresses
#'
#' # Get latest assessment for subspecies
#' ex2 <- rl_species_latest(genus = "Gorilla", species = "gorilla",
#'                          infra = "gorilla")
#' ex2$stresses
#' }
rl_species_latest <- function(genus, species, infra = NULL,
                              subpopulation = NULL, scope = "1",
                              key = NULL, parse = TRUE, ...) {
  assert_is(parse, "logical")
  assert_is(scope, c("character"))

  tmp <- rl_species(genus, species, infra = infra,
                    subpopulation = subpopulation, key, ...)$assessments
  if (!is.null(scope)) {
    ind <- vapply(tmp$scopes, function(x) scope %in% x$code, logical(1))
    if (any(ind)) {
      tmp <- tmp[ind, , drop = FALSE]
    } else {
      cli_alert_warning(paste(
        "No assessment(s) found for the specified scope.",
        "Returning the latest assessment across all scopes.")
      )
    }
  }
  if (any(tmp$latest, na.rm = TRUE)) {
    tmp <- subset(tmp, tmp$latest)
  }
  tmp$year_published <- as.numeric(as.character(tmp$year_published))
  ord <- order(tmp$year_published, decreasing = TRUE)
  tmp <- tmp[ord, , drop = FALSE]
  rl_assessment(id = tmp$assessment_id[1], key = key, parse = parse, ...)
}

#' Family assessment summary
#'
#' Get an assessment summary for a particular family
#'
#' @export
#' @param family (character) The name of the family to look up. If not supplied,
#'   a list of all family names will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family taxa
#' @examples \dontrun{
#' # Get assessment summary for family
#' ex1 <- rl_family(family = "Hominidae")
#' nrow(ex1$assessments)
#' }
rl_family <- function(family = NULL, key = NULL, parse = TRUE, all = TRUE,
                      page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_family_(family, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_family
rl_family_ <- function(family = NULL, key = NULL, all = TRUE, page = 1,
                       quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(family, "character")
  assert_n(family, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("taxa/family", family, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Order assessment summary
#'
#' Get an assessment summary for a particular order
#'
#' @export
#' @param order (character) The name of the order to look up. If not supplied, a
#'   list of all order names will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family taxa
#' @examples \dontrun{
#' # Get assessment summary for order
#' ex1 <- rl_order(order = "Apiales")
#' nrow(ex1$assessments)
#' }
rl_order <- function(order = NULL, key = NULL, parse = TRUE, all = TRUE,
                     page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_order_(order, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_order
rl_order_ <- function(order = NULL, key = NULL, all = TRUE, page = 1,
                      quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(order, "character")
  assert_n(order, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("taxa/order", order, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Class assessment summary
#'
#' Get an assessment summary for a particular class
#'
#' @export
#' @param class (character) The name of the class to look up. If not supplied, a
#'   list of all class names will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family taxa
#' @examples \dontrun{
#' # Get assessment summary for class
#' ex1 <- rl_class(class = "Mammalia")
#' nrow(ex1$assessments)
#' }
rl_class <- function(class = NULL, key = NULL, parse = TRUE, all = TRUE,
                     page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_class_(class, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_class
rl_class_ <- function(class = NULL, key = NULL, all = TRUE, page = 1,
                      quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(class, "character")
  assert_n(class, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("taxa/class", class, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Phylum assessment summary
#'
#' Get an assessment summary for a particular phylum
#'
#' @export
#' @param phylum (character) The name of the phylum to look up. If not supplied,
#'   a list of all phylum names will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family taxa
#' @examples \dontrun{
#' # Get assessment summary for phylum
#' ex1 <- rl_phylum(phylum = "Annelida`")
#' nrow(ex1$assessments)
#' }
rl_phylum <- function(phylum = NULL, key = NULL, parse = TRUE, all = TRUE,
                      page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_phylum_(phylum, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_phylum
rl_phylum_ <- function(phylum = NULL, key = NULL, all = TRUE, page = 1,
                       quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(phylum, "character")
  assert_n(phylum, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("taxa/phylum", phylum, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Kingdom assessment summary
#'
#' Get an assessment summary for a particular kingdom
#'
#' @export
#' @param kingdom (character) The name of the kingdom to look up. If not
#'   supplied, a list of all kingdom names will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family taxa
#' @examples \dontrun{
#' # Get assessment summary for kingdom
#' ex1 <- rl_kingdom(kingdom = "Fungi")
#' nrow(ex1$assessments)
#' }
rl_kingdom <- function(kingdom = NULL, key = NULL, parse = TRUE, all = TRUE,
                       page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_kingdom_(kingdom, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_kingdom
rl_kingdom_ <- function(kingdom = NULL, key = NULL, all = TRUE, page = 1,
                        quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(kingdom, "character")
  assert_n(kingdom, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("taxa/kingdom", kingdom, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Extinct taxa assessment summary
#'
#' Get an assessment summary for all possibly extinct taxa
#'
#' @export
#' @template all
#' @template curl
#' @template info
#' @template page
#' @family groups
#' @examples \dontrun{
#' ex1 <- rl_extinct()
#' nrow(ex1$assessments)
#' }
rl_extinct <- function(key = NULL, parse = TRUE, all = TRUE, page = 1,
                       quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_extinct_(key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_extinct
rl_extinct_ <- function(key = NULL, all = TRUE, page = 1, quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- "taxa/possibly_extinct"
  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Extinct taxa in the wild assessment summary
#'
#' Get an assessment summary for all taxa that are possibly extinct in the wild
#'
#' @export
#' @template all
#' @template curl
#' @template info
#' @template page
#' @family groups
#' @examples \dontrun{
#' ex1 <- rl_extinct_wild()
#' nrow(ex1$assessments)
#' }
rl_extinct_wild <- function(key = NULL, parse = TRUE, all = TRUE, page = 1,
                            quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_extinct_wild_(key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_extinct_wild
rl_extinct_wild_ <- function(key = NULL, all = TRUE, page = 1,
                             quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- "taxa/possibly_extinct_in_the_wild"
  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

context("package fails well")

test_that("fails well when API is down", {
  skip_on_cran()
  webmockr::enable()
  stub <- webmockr::stub_request("get", paste0(rr_base(),
                                               "/taxa/scientific_name?",
                                               "genus_name=Loxodonta&",
                                               "species_name=africana"))
  webmockr::to_return(stub, status = 503)
  expect_error(rl_species("Loxodonta", "africana"),
               "The IUCN API is not currently available")
  webmockr::disable()
})

test_that("fails well on bad key", {
  skip_on_cran()

  vcr::use_cassette("rl_species-badkey", {
    expect_error(rl_species("Loxodonta", "africana", key = "adfds"),
                 "Token not valid!")
  })
  vcr::use_cassette("rl_species_-badkey", {
    expect_error(rl_species_("Loxodonta", "africana", key = "adfds"),
                 "Token not valid")
  })
})

test_that("fails well on bad query", {
  skip_on_cran()

  vcr::use_cassette("rl_species-badquery", {
    expect_error(rl_species("Loxodonta", "africanum"),
                 "No results returned for query")
  })
  vcr::use_cassette("rl_species_-badquery", {
    expect_error(rl_species_("Loxodonta", "africanum"),
                 "No results returned for query")
  })
})

test_that("fails well for other status codes", {
  skip_on_cran()
  webmockr::enable()
  stub <- webmockr::stub_request("get", paste0(rr_base(),
                                               "/taxa/scientific_name?",
                                               "genus_name=Loxodonta&",
                                               "species_name=africana"))
  webmockr::to_return(stub, status = 418)
  expect_error(rl_species("Loxodonta", "africana"),
               "I'm a teapot")
  webmockr::disable()
})

test_that("fails well when correct parameters not given", {
  skip_on_cran()

  expect_error(rl_species(), "is missing, with no default")
  expect_error(rl_species_(), "is missing, with no default")

  expect_error(rl_species(c("Gorilla", "Gorilla"), c("gorilla", "gorilla")),
               "must be length 1")
  expect_error(rl_species_(c("Gorilla", "Gorilla"), c("gorilla", "gorilla")),
               "must be length 1")
})

context("rl_latest functions")

test_that("rl_species_latest works", {
  skip_on_cran()

  vcr::use_cassette("rl_species_latest", {
    aa <- rl_species_latest("Gorilla", "gorilla")
  })

  expect_is(aa, "list")
  expect_is(aa$taxon, "list")
  expect_is(aa$habitats, "data.frame")

  vcr::use_cassette("rl_species_latest_global", {
    bb <- rl_species_latest("Canis", "lupus", scope = "1")
  })

  vcr::use_cassette("rl_species_latest_europe", {
    cc <- rl_species_latest("Canis", "lupus", scope = "2")
  })

  expect_false(identical(bb$assessment_id, cc$assessment_id))
})

test_that("rl_sis_latest works", {
  skip_on_cran()

  vcr::use_cassette("rl_sis_latest", {
    aa <- rl_sis_latest(9404)
  })

  expect_is(aa, "list")
  expect_is(aa$taxon, "list")
  expect_is(aa$habitats, "data.frame")

  vcr::use_cassette("rl_sis_latest_global", {
    bb <- rl_sis_latest(3746, scope = "1")
  })

  vcr::use_cassette("rl_sis_latest_europe", {
    cc <- rl_sis_latest(3746, scope = "2")
  })

  expect_false(identical(bb$assessment_id, cc$assessment_id))
})

test_that("rl_sis_latest works with no latest assessment", {
  skip_on_cran()

  vcr::use_cassette("rl_sis_latest-no-latest-assessment", {
    aa <- rl_sis_latest(41129)
  })

  expect_is(aa, "list")
  expect_is(aa$taxon, "list")
})

test_that("rl_species_latest fails well", {
  skip_on_cran()

  expect_error(rl_species_latest(), "is missing, with no default")
  expect_error(rl_species_latest(5), "genus must be of class character")
  expect_error(rl_species_latest(list()), "genus must be of class character")

  expect_error(rl_species_latest("Gorilla", 5),
               "species must be of class character")
  expect_error(rl_species_latest("Gorilla", list()),
               "species must be of class character")

  expect_error(rl_species_latest("Gorilla", "gorilla", key = 5),
               "key must be of class character")
  expect_error(rl_species_latest("Gorilla", "gorilla", key = matrix()),
               "key must be of class character")

  expect_error(rl_species_latest("Gorilla", "gorilla", parse = 5),
               "parse must be of class logical")
  expect_error(rl_species_latest("Gorilla", "gorilla", parse = matrix()),
               "parse must be of class logical")

  vcr::use_cassette("rl_species_latest_wrong_scope", {
    expect_message(rl_species_latest("Gorilla", "gorilla", scope = "2"),
                   "No assessment")
  })
})

test_that("rl_sis_latest fails well", {
  skip_on_cran()

  expect_error(rl_sis_latest(), "is missing, with no default")

  expect_error(rl_sis_latest(9404, key = 5),
               "key must be of class character")
  expect_error(rl_sis_latest(9404, key = matrix()),
               "key must be of class character")

  expect_error(rl_sis_latest(9404, parse = 5),
               "parse must be of class logical")
  expect_error(rl_sis_latest(9404, parse = matrix()),
               "parse must be of class logical")

  vcr::use_cassette("rl_sis_latest_wrong_scope", {
    expect_message(rl_sis_latest(9404, scope = "2"),
                   "No assessment")
  })
})

context("rl_sis functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_sis", {
    aa <- rl_sis(9404)
  })

  expect_is(aa, "list")
  expect_named(aa, c("sis_id", "taxon", "assessments"))
  expect_is(aa$sis_id, "integer")
  expect_is(aa$taxon, "list")
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_sis-not-parsing", {
    aa <- rl_sis(9404, parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("sis_id", "taxon", "assessments"))
  expect_is(aa$sis_id, "integer")
  expect_is(aa$taxon, "list")
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_sis_", {
    aa <- rl_sis_(9404, all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("sis_id", "taxon", "assessments"))
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_sis(), "is missing, with no default")

  expect_error(rl_sis("5"), "id must be of class integer")
  expect_error(rl_sis(list()), "id must be of class integer")

  expect_error(rl_sis(9404, key = 5),
               "key must be of class character")
  expect_error(rl_sis(9404, key = matrix()),
               "key must be of class character")
})

context("rl_species functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_species", {
    aa <- rl_species("Gorilla", "gorilla")
  })

  expect_is(aa, "list")
  expect_named(aa, c("taxon", "assessments", "params"))
  expect_is(aa$taxon, "list")
  expect_is(aa$assessments, "data.frame")
  expect_is(aa$params, "list")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_species-not-parsing", {
    aa <- rl_species("Gorilla", "gorilla", parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("taxon", "assessments", "params"))
  expect_is(aa$taxon, "list")
  expect_is(aa$assessments, "list")
  expect_is(aa$params, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_species_", {
    aa <- rl_species_("Gorilla", "gorilla", all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("taxon", "assessments", "params"))
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_species(), "is missing, with no default")

  expect_error(rl_species(5), "genus must be of class character")
  expect_error(rl_species(list()), "genus must be of class character")

  expect_error(rl_species("Gorilla", 5), "species must be of class character")
  expect_error(rl_species("Gorilla", list()),
               "species must be of class character")

  expect_error(rl_species("Gorilla", "gorilla", key = 5),
               "key must be of class character")
  expect_error(rl_species("Gorilla", "gorilla", key = matrix()),
               "key must be of class character")

  expect_error(rl_species("Gorilla", "gorilla", parse = 5),
               "parse must be of class logical")
  expect_error(rl_species("Gorilla", "gorilla", parse = matrix()),
               "parse must be of class logical")

  expect_error(rl_species("Gorilla", "gorilla", infra = 5),
               "infra must be of class character")
  expect_error(rl_species("Gorilla", "gorilla", subpopulation = 2),
               "subpopulation must be of class character")
})

context("rl_extinct_wild functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_extinct_wild", {
    aa <- rl_extinct_wild(all = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("assessments"))
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_extinct_wild-not-parsing", {
    aa <- rl_extinct_wild(all = FALSE, parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("assessments"))
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_extinct_wild_", {
    aa <- rl_extinct_wild_(all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("assessments"))
  expect_is(aajson$assessments, "data.frame")
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_extinct_wild(key = 5), "key must be of class character")
  expect_error(rl_extinct_wild(key = matrix()),
               "key must be of class character")

  expect_error(rl_extinct_wild(parse = 5), "parse must be of class logical")
  expect_error(rl_extinct_wild(parse = matrix()),
               "parse must be of class logical")

  expect_error(rl_extinct_wild(page = "next"), "page must be of class integer")
  expect_error(rl_extinct_wild(all = "yes"), "all must be of class logical")
  expect_error(rl_extinct_wild(quiet = "no"), "quiet must be of class logical")

  # lengths
  expect_error(rl_extinct_wild(page = 1:2), "page must be length 1")
})

#' Retrieve an assessment
#'
#' Get the full details for a single IUCN Red List assessment.
#'
#' @export
#' @param id (integer) The unique identifier of the assessment.
#' @template all
#' @template curl
#' @template info
#' @examples \dontrun{
#' # Get assessment details for Fratercula arctica
#' ex1 <- rl_assessment(id = 166290968)
#' ex1$red_list_category$code
#' ex1$systems
#' }
rl_assessment <- function(id, key = NULL, parse = TRUE, ...) {
  assert_is(parse, "logical")

  rl_parse(rl_assessment_(id, key, ...), parse)
}

#' @export
#' @rdname rl_assessment
rl_assessment_ <- function(id, key = NULL, ...) {
  assert_is(key, "character")
  assert_is(id, c("integer", "numeric"))
  assert_n(id, 1)

  rr_GET(paste0("assessment/", id), key, ...)
}

#' Retrieve a list of assessments
#'
#' Get the full details for a list of IUCN Red List assessments. This is a
#' wrapper for [rl_assessment()] that allows you to retrieve multiple
#' assessments at once.
#'
#' @param ids (integer) A vector of unique identifiers of the assessments.
#' @param key (character) An IUCN API token. See [rl_use_iucn()].
#' @param quiet (logical) If \code{TRUE}, suppresses the progress bar.
#' @param wait_time (numeric) The number of seconds to wait between queries. The
#'   default is 0.5 seconds, which is recommended by IUCN to avoid timeouts.
#' @template curl
#' @export
#' @return A list with each element representing the response of
#'   [rl_assessment()].
#'
#' @examples \dontrun{
#' # Get assessment details for multiple assessments
#' ex1 <- rl_assessment_list(ids = c(166290968, 136250858))
#' }
#' @importFrom cli cli_alert_success cli_alert_warning cli_progress_bar
#' @importFrom cli cli_progress_update cli_progress_done
rl_assessment_list <- function(ids, key = NULL, wait_time = 0.5, quiet = FALSE,
                               ...) {
  assert_is(key, "character")
  assert_is(ids, c("integer", "numeric"))
  assert_is(wait_time, c("integer", "numeric"))
  assert_is(quiet, "logical")
  if (wait_time < 0.5) {
    cli_alert_warning(paste("Waiting for", wait_time, "seconds between API",
                            "calls."))
    cli_alert_warning(paste("This is a short wait time and could result in",
                            "your API token being rate limited."))
    cli_alert_warning(paste("IUCN recommends wait times >=0.5 seconds between",
                            "calls to maintain service reliability."))
  } else {
    cli_alert_success(paste("Waiting for", wait_time, "second(s) between",
                            "API calls."))
  }
  if (!quiet) {
    prog_id <- cli_progress_bar(
      "Querying assessments", total = length(ids), clear = FALSE,
      format = paste0("{cli::pb_name} ({cli::pb_current}/{cli::pb_total}) | ",
                      "{cli::pb_bar} {cli::pb_percent} | ETA: {cli::pb_eta}"))
  }
  lst <- lapply(ids,
                function(id) {
                  Sys.sleep(wait_time)
                  tryCatch({
                    rl_assessment(id, key = key, ...)
                  }, error = function(e) {
                    cli_alert_warning(
                      paste(
                        "Couldn't find an IUCN assessment with {.field ID}",
                        "{.val {id}}."
                      )
                    )
                    return(NULL)
                  }, finally = if (!quiet) cli_progress_update(id = prog_id))
                }
  )
  if (!quiet) cli_progress_done(id = prog_id)
  return(lst)
}

#' Extract an assessment element
#'
#' Extract a given element from each of a list of assessments, such as the
#' output of [rl_assessment_list()]. This is useful for extracting specific
#' details from the assessments, such as taxonomy (`el_name = "taxon"`),
#' synonyms (`el_name = "taxon__synonyms"`), habitats (`el_name = "habitats"`),
#' or the red list category (`el_name = "red_list_category"`).
#'
#' @details The following top-level element names can be extracted as of
#'   `r Sys.Date()`:
#'   \itemize{
#'     `r get_assessment_elements()`
#'   }
#'
#'   Note that there are also sublevels of the assessment hierarchy, but they
#'   would be unwieldy to display here. A complete and up-to-date list can be
#'   found by inspecting the return of an [rl_assessment()] call.
#'
#' @param lst (list) A list of assessments, as returned by
#'   [rl_assessment_list()]. If `lst` contains any `NULL` elements, they will be
#'   removed.
#' @param el_name (character) The name of the element to extract from each
#'   assessment. Supports multilevel extraction using "__" as the separator. For
#'   example, to extract the synonyms table, you could use "taxon__synonyms".
#'   `el_name` may also be a character vector indicating multiple elements to
#'   extract. In this case, the output will contain all requested elements,
#'   either as a list (if `format = "list"`) or as a data.frame (if `format =
#'   "df"`). In the case of a data.frame, the extracted elements will be joined
#'   by `assessment_id`, which may result in an unexpected merged outcome when
#'   multiple rows are present in one or more of the extracted elements (e.g.,
#'   the `"threats"` element).
#' @param format (character) The format of the output. Either "list" or "df"
#'   (for a data.frame).
#' @param flatten (logical) If `TRUE`, the output will be flattened to a
#'   data.frame. Note that this may not work for all elements, especially
#'   complex multilevel list elements. Only used when `format = "df"`. The
#'   `dplyr`, `tidyr`, and `tibble` packages are required to use this feature.
#'   Note that fields with no data across all assessments may be lost.
#' @return
#'   A list or data.frame containing the extracted element from each
#'   assessment.
#' @export
#' @importFrom rlang check_installed expr
#' @importFrom cli cli_abort
#' @importFrom stats setNames
#' @examples \dontrun{
#' lst <- rl_assessment_list(ids = c(166290968, 136250858))
#' # get complex elements as a list
#' ex1 <- rl_assessment_extract(lst, "taxon")
#' # get simple elements as a data.frame
#' ex2 <- rl_assessment_extract(lst, "red_list_category__code", format = "df")
#' # get complex elements as a data.frame
#' ex3 <- rl_assessment_extract(lst, "threats", format = "df")
#' # get the same elements flattened to a single data.frame
#' ex4 <- rl_assessment_extract(lst, "threats", format = "df", flatten = TRUE)
#' # get subelements flattened to a data.frame
#' ex5 <- rl_assessment_extract(lst, "taxon__order_name", format = "df",
#'                              flatten = TRUE)
#' # get a data frame with both taxon name and red list category code
#' ex6 <- rl_assessment_extract(lst, c("taxon", "red_list_category__code"),
#'                              format = "df", flatten = TRUE)
#' }
rl_assessment_extract <- function(lst, el_name, format = c("list", "df"),
                                  flatten = FALSE) {
  assert_is(lst, "list")
  assert_is(el_name, "character")
  assert_is(format, "character")
  assert_is(flatten, "logical")
  format <- match.arg(format)
  # filter out NULL elements
  lst <- Filter(Negate(is.null), lst)
  # get assessment ids for later
  ids <- vapply(lst, function(x) x$assessment_id, FUN.VALUE = integer(1))
  for (el_ind in seq_along(el_name)) {
    lst_tmp <- lst
    # extract levels of extraction
    el_name_spl <- strsplit(el_name[el_ind], "__")[[1]]

    # perform multi-level extraction
    for (el in el_name_spl) {
      # check if element exists
      if (!any(sapply(lst_tmp, function(x) el %in% names(x)))) {
        cli_abort(
          paste("Element {.val {el}} not found in any of the assessments.")
        )
      }
      lst_tmp <- lapply(lst_tmp, function(x) x[[el]])
    }

    # put output in requested format
    if (format == "list") {
      names(lst_tmp) <- as.character(ids)
      if (el_ind == 1) {
        ret_obj <- lst_tmp
      } else (
        ret_obj <- mapply(c, ret_obj, lst_tmp, SIMPLIFY = FALSE)
      )
    } else {
      if (flatten) {
        check_installed(c("dplyr", "tibble", "tidyr"),
                        reason = "to use `flatten = TRUE`")
        tryCatch({
          for (i in seq_along(lst_tmp)) {
            if (is.list(lst_tmp[[i]]) && !is.data.frame(lst_tmp[[i]])) {
              # need to handle all sorts of formats, otherwise would use as_tibble
              lst_tmp[[i]] <- tidyr::pivot_wider(tibble::enframe(lst_tmp[[i]]),
                                             names_from = "name",
                                             values_from = "value")
            } else {
              lst_tmp[[i]] <- as.data.frame(lst_tmp[[i]])
            }
            # if there is only one column, rename it to the element name
            if (ncol(lst_tmp[[i]]) == 1) {
              colnames(lst_tmp[[i]]) <- el
            }
            if (nrow(lst_tmp[[i]]) > 0) {
              lst_tmp[[i]]$assessment_id <- ids[i]
            }
          }
          df <- dplyr::bind_rows(lst_tmp)
        },
        error = function(e) {
          cli_abort(# nocov start
            paste("Error flattening the {.val {el_name}} element to a",
                  "data.frame. Try setting {.code flatten = FALSE} or extracting",
                  "a lower level element of the assessments."),
            call = expr(rl_assessment_extract())
          )# nocov end
        })

        # move assessment_id to first column
        df <- df[, c("assessment_id", setdiff(names(df), "assessment_id"))]
      } else {
        df <- data.frame(assessment_id = ids)
        df[[el]] <- lst_tmp
      }
      # convert complex columns
      for (col in colnames(df)) {
        if (
          # if data frame column, then append as additional columns
          is.data.frame(df[[col]]) &&
          all(sapply(df[[col]], Negate(is.list))) &&
          identical(nrow(df[[col]]), nrow(df))
        ) {
          # prepare new column(s)
          # columns are renamed to avoid potential clashes
          # e.g., if df contains a data frame column called "description" with a
          # column called "en" (e.g., df$description$en), then this column
          # will be appended as df$description.en
          new_col <- setNames(df[[col]], paste0(col, ".", colnames(df[[col]])))
          # identify the index to insert new column(s)
          idx <- which(colnames(df) == col)
          # insert new columns based on the index, and drop data frame column
          if (identical(idx, 1L)) {
            df <- cbind(new_col, df[, -idx, drop = FALSE])
          } else if (identical(idx, ncol(df))) {
            df <- cbind(df, df[, -idx, drop = FALSE])
          } else {
            df <- cbind(
              df[, seq(1, idx - 1), drop = FALSE],
              new_col,
              df[, seq(idx + 1, ncol(df)), drop = FALSE]
            )
          }
        } else if (
          # if list column with atomic vector of values, then append as column
          is.list(df[[col]]) &&
          all(vapply(df[[col]], Negate(is.list), logical(1)))
        ) {
          # replace NULL with NA to prevent breakage with unlist()
          tmp <- df[[col]]
          tmp[vapply(tmp, is.null, logical(1))] <- NA
          df[[col]] <- unlist(tmp, use.names = FALSE)
        }
      }
      if (el_ind == 1) {
        ret_obj <- df
      } else (
        ret_obj <- dplyr::left_join(ret_obj, df, by = "assessment_id")
      )
    }
  }
  return(ret_obj)
}
# nocov start
get_assessment_elements <- function() {
  element_names <- names(rl_assessment(id = 166290968))
  return(paste0("\\item ", sort(element_names), collapse = "\n"))
}
# nocov end

context("rl_assessment functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_assessment", {
    aa <- rl_assessment(166290968)
  })

  expect_is(aa, "list")
  expect_is(aa$taxon, "list")
  expect_is(aa$habitats, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_assessment-not-parsing", {
    aa <- rl_assessment(166290968, parse = FALSE)
  })

  expect_is(aa, "list")
  expect_is(aa$taxon, "list")
  expect_is(aa$habitats, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_assessment_", {
    aa <- rl_assessment_(166290968)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_is(aajson$taxon, "list")
  expect_is(aajson$habitats, "data.frame")
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_assessment(), "is missing, with no default")

  expect_error(rl_assessment(166290968, key = 5),
               "key must be of class character")
  expect_error(rl_assessment(166290968, key = matrix()),
               "key must be of class character")

  expect_error(rl_assessment(166290968, parse = 5),
               "parse must be of class logical")
  expect_error(rl_assessment(166290968, parse = matrix()),
               "parse must be of class logical")
})

test_that("rl_assessment_list works", {
  skip_on_cran()

  vcr::use_cassette("rl_assessment_list", {
    aa <- rl_assessment_list(ids = c(166290968, 136250858))
  })

  vcr::use_cassette("rl_assessment_list_check1", {
    bb <- rl_assessment(166290968)
  })
  vcr::use_cassette("rl_assessment_list_check2", {
    cc <- rl_assessment(136250858)
  })

  vcr::use_cassette("rl_assessment_list_warning", {
    expect_message(rl_assessment_list(ids = c(166290968, 136250858),
                                      wait_time = 0), "short wait time")
  })

  vcr::use_cassette("rl_assessment_list_warning2", {
    expect_message(rl_assessment_list(ids = c(5, 136250858)), "Couldn't find")
  })

  expect_is(aa, "list")
  expect_length(aa, 2)
  expect_equal(aa[[1]], bb)
  expect_equal(aa[[2]], cc)

  expect_error(rl_assessment_list(), "is missing, with no default")
  expect_error(rl_assessment_list(ids = ""),
               "ids must be of class integer")
  expect_error(rl_assessment_list(166290968, wait_time = "never"),
               "wait_time must be of class integer")
  expect_error(rl_assessment_list(166290968, quiet = "yes"),
               "quiet must be of class logical")
})

test_that("rl_assessment_extract works", {
  skip_on_cran()

  vcr::use_cassette("rl_assessment_list_for_extract", {
    aa <- rl_assessment_list(ids = c(166290968, 136250858))
  })

  extract <- rl_assessment_extract(aa, "taxon")
  expect_is(extract, "list")
  expect_length(extract, 2)
  expect_is(extract[[1]], "list")

  extract2 <- rl_assessment_extract(aa, "taxon__common_names", format = "df")
  expect_is(extract2, "data.frame")
  expect_equal(nrow(extract2), 2)
  expect_is(extract2$common_names, "list")
  expect_equal(colnames(extract2)[1], "assessment_id")

  extract3 <- rl_assessment_extract(aa, "taxon__common_names", format = "df",
                                    flatten = TRUE)
  expect_is(extract3, "data.frame")
  expect_equal(nrow(extract3), sum(nrow(extract2$common_names[[1]]),
                                   nrow(extract2$common_names[[2]])))

  extract4 <- rl_assessment_extract(aa, "threats", format = "df",
                                    flatten = TRUE)
  expect_is(extract4, "data.frame")

  extract5 <- rl_assessment_extract(aa, "taxon__kingdom_name", format = "df")
  expect_is(extract5, "data.frame")
  expect_named(extract5, c("assessment_id", "kingdom_name"))

  extract6 <- rl_assessment_extract(aa, "taxon__kingdom_name", format = "df",
                                    flatten = TRUE)
  expect_is(extract6, "data.frame")
  expect_named(extract6, c("assessment_id", "kingdom_name"))
  expect_equal(extract5, extract6)

  extract7 <- rl_assessment_extract(aa, "taxon", format = "df", flatten = TRUE)
  expect_is(extract7, "data.frame")
  expect_equal(colnames(extract7)[1], "assessment_id")
  expect_equal(extract7$kingdom_name, extract6$kingdom_name)

  # test with a vector of elements to extract
  extract8 <- rl_assessment_extract(aa, c("taxon", "threats"))
  expect_is(extract8, "list")
  expect_length(extract8, 2)
  expect_is(extract8[[1]], "list")
  expect_is(extract8[[2]], "list")
  expect_true("kingdom_name" %in% names(extract8[[1]]))
  expect_true("code" %in% names(extract8[[1]]))

  extract9 <- rl_assessment_extract(aa, c("taxon", "threats"),
                                    format = "df", flatten = TRUE)
  expect_is(extract9, "data.frame")
  expect_equal(colnames(extract9)[1], "assessment_id")
  expect_true("kingdom_name" %in% colnames(extract9))
  expect_true("code" %in% colnames(extract9))

  expect_error(rl_assessment_extract(), "is missing, with no default")
  expect_error(rl_assessment_extract(lst = 166290968, el_name = "taxon"),
               "lst must be of class list")
  expect_error(rl_assessment_extract(aa, el_name = 5),
               "el_name must be of class character")
  expect_error(rl_assessment_extract(aa, el_name = "abcdefg"),
               "not found")
  expect_error(rl_assessment_extract(aa, c("taxon", "abcdefg")),
               "not found")
  expect_error(rl_assessment_extract(aa, el_name = "taxon", format = TRUE),
               "format must be of class character")
  expect_error(rl_assessment_extract(aa, el_name = "taxon", format = "test"))
  expect_error(rl_assessment_extract(aa, el_name = "taxon", flatten = "yes"),
               "flatten must be of class logical")
})

#' Biogeographical realm assessment summary
#'
#' Get an assessment summary for a particular biogeographical realm (e.g.,
#' Neotropical or Palearctic).
#'
#' @export
#' @param code (character) The code of the biogeographical realm to look up. If
#'   not supplied, a list of all biogeographical realms will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family geo
#' @examples \dontrun{
#' # Get a list of all biogeographical realms
#' rl_realms()
#' # Get assessment summary for the Afrotropical biogeographical realm
#' rl_realms(code = "0")
#' }
rl_realms <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                      page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_realms_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_realms
rl_realms_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                       quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("biogeographical_realms", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Scope assessment summary
#'
#' Return assessments for a given geographical assessment scope (e.g., Global,
#' Mediterranean). This is similar to the `region` argument of the old Red List
#' API and old versions of rredlist.
#'
#' @export
#' @param code (character) The code of the scope to look up. If not supplied, a
#'   list of all scopes will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family geo
#' @examples \dontrun{
#' # Get a list of all scopes
#' rl_scopes()
#' # Get assessment summary for the Gulf of Mexico scope
#' rl_scopes(code = "45433062")
#' }
rl_scopes <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                      page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_scopes_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_scopes
rl_scopes_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                       quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("scopes", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' FAO marine fishing region assessment summary
#'
#' Return assessments for a given marine fishing region as defined by the Food
#' and Agriculture Organization (FAO) of the United Nations (e.g., Pacific -
#' northwest, Arctic Sea). More details are available here:
#' <https://www.fao.org/fishery/en/area>.
#'
#' @export
#' @param code (character) The code of the FAO region to look up. If not
#'   supplied, a list of all FAO regions will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family geo
#' @examples \dontrun{
#' # Get a list of all scopes
#' rl_faos()
#' # Get assessment summary for the Arctic Sea FAO region
#' rl_faos(code = "18")
#' }
rl_faos <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                    page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_faos_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_faos
rl_faos_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                     quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("faos", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

context("rl_faos functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_faos-def", {
    aa <- rl_faos()
  })

  expect_is(aa, "list")
  expect_named(aa, c("faos"))
  expect_is(aa$faos, "data.frame")

  vcr::use_cassette("rl_faos", {
    aa <- rl_faos("18")
  })

  expect_is(aa, "list")
  expect_named(aa, c("fao", "assessments", "filters"))
  expect_is(aa$fao, "list")
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_faos-def-not-parsing", {
    aa <- rl_faos(parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("faos"))
  expect_is(aa$faos, "list")

  vcr::use_cassette("rl_faos-not-parsing", {
    aa <- rl_faos("18", parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("fao", "assessments", "filters"))
  expect_is(aa$fao, "list")
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_faos_-def", {
    aa <- rl_faos_()
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("faos"))
  expect_is(aajson$faos, "data.frame")

  vcr::use_cassette("rl_faos_", {
    aa <- rl_faos_("18", all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("fao", "assessments", "filters"))
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_faos(5), "code must be of class character")
  expect_error(rl_faos(list()), "code must be of class character")

  expect_error(rl_faos(key = 5), "key must be of class character")
  expect_error(rl_faos(key = matrix()), "key must be of class character")

  expect_error(rl_faos(parse = 5), "parse must be of class logical")
  expect_error(rl_faos(parse = matrix()), "parse must be of class logical")

  expect_error(rl_faos(page = "next"), "page must be of class integer")
  expect_error(rl_faos(all = "yes"), "all must be of class logical")
  expect_error(rl_faos(quiet = "no"), "quiet must be of class logical")

  # lengths
  expect_error(rl_faos(page = 1:2), "page must be length 1")

})

#' Helper to get and save IUCN API key
#'
#' @description Provides instruction on how to get and store a key for the IUCN
#'   API. Users of `rredlist` must do this before using any of the package's
#'   other functionality. Note that registering for a key requires an email
#'   address and information about your organization.
#'
#' @details Note that after filling the online form, you should receive an API
#'   key shortly but not immediately.
#'
#' @aliases rl_key
#'
#' @export
#' @return Invisibly returns the sign-up URL for the IUCN Red List API.
#' @importFrom cli cli_h2 cli_ol cli_li cli_code cli_end
#' @examples \dontrun{
#' # Sign up for an API key
#' rl_use_iucn()
#' }
rl_use_iucn <- function() {
  cli_h2("To get and save an IUCN API key, follow these steps:")
  cli_ol()
  cli_li("Register for an API key on the {.href [IUCN website](https://api.iucnredlist.org/users/sign_up)} (requires an email address and organization info)")
  cli_li("Open and edit your .Renviron file (e.g., run {.run usethis::edit_r_environ()})")
  cli_li("Add the following text to your .Renviron file, inserting your own API key value:")
  cli_code("     IUCN_REDLIST_KEY='youractualkeynotthisstring'")
  cli_li("Restart R")
  cli_end()

  invisible("https://api.iucnredlist.org/users/sign_up")
}

context("rl_key functions")

test_that("rl_key produces expected URL and message", {
  suppressMessages(expect_equal(rl_use_iucn(),
                                "https://api.iucnredlist.org/users/sign_up"))
  expect_message(rl_use_iucn(), "To get and save an IUCN API key")
})

# set up vcr
library("vcr")

vcr_dir <- vcr::vcr_test_path("fixtures")

if (!nzchar(Sys.getenv("IUCN_REDLIST_KEY"))) {
  if (dir.exists(vcr_dir)) {
    Sys.setenv("IUCN_REDLIST_KEY" = "foobar")
  } else {
    stop("No API key nor cassettes, tests cannot be run.",
         call. = FALSE)
  }
}

invisible(vcr::vcr_configure(
  dir = vcr_dir,
  filter_sensitive_data = list("<<rredlist_api_token>>" =
                                 Sys.getenv("IUCN_REDLIST_KEY")),
  filter_request_headers = list(Authorization = "My bearer token is safe")
))

# load other needed packages
library("jsonlite")
library("webmockr")

#' Filter a string to non-null elements
#' @param l A list.
#' @noRd
#' @return A list.
ct <- function(l) Filter(Negate(is.null), l)

#' Get user agent info
#' @noRd
#' @return A string indicating the package version numbers for the curl, crul,
#'   and rredlist R packages.
#' @importFrom utils packageVersion
rredlist_ua <- function() {
  versions <- c(
    paste0("r-curl/", packageVersion("curl")),
    paste0("crul/", packageVersion("crul")),
    sprintf("rOpenSci(rredlist/%s)", packageVersion("rredlist"))
  )
  paste0(versions, collapse = " ")
}

#' Build and make a GET query of the IUCN API
#'
#' @param path (character) The full API endpoint.
#' @param key (character) An IUCN API token. See [rl_use_iucn()].
#' @param query (list) A list of parameters to include in the GET query.
#' @param ... [Curl options][curl::curl_options()] passed to the GET request via
#'   [HttpClient][crul::HttpClient()].
#'
#' @noRd
#' @return The raw crul response object.
#' @importFrom crul HttpClient
rr_GET_raw <- function(path, key = NULL, query = list(), ...) {
  # Extract secret API query arguments
  args <- list(...)
  query$latest <- args$latest
  query$scope_code <- args$scope_code
  query$year_published <- args$year_published

  check_internet()
  cli <- HttpClient$new(
    url = paste(rr_base(), space(path), sep = "/"),
    opts = list(useragent = rredlist_ua()),
    headers = list(Authorization = check_key(key))
  )
  cli$retry("GET", query = ct(query), ..., retry_only_on = c(429))
}

#' Handle a GET query of the IUCN API
#'
#' @param path (character) The full API endpoint.
#' @param key (character) An IUCN API token. See [rl_use_iucn()].
#' @param query (list) A list of parameters to include in the GET query.
#' @param ... [Curl options][curl::curl_options()] passed to the GET request via
#'   [HttpClient][crul::HttpClient()].
#'
#' @noRd
#' @return The response of the query as a JSON string.
#' @importFrom crul HttpClient
rr_GET <- function(path, key = NULL, query = list(), ...) {
  res <- rr_GET_raw(path, key, query, ...)
  status_catcher(res)
  x <- res$parse("UTF-8")
  return(x)
}

#' Check that the user has internet
#'
#' @return If the user has internet, nothing is returned. If the user does not
#'   have internet, an error is thrown.
#' @noRd
#' @importFrom curl nslookup
check_internet <- function() {
  # check for internet with DNS lookup
  if (is.null(nslookup("google.com", error = FALSE))) {
    stop("An internet connection is required to access the IUCN API.")
  }
}

#' Catch status code errors
#' @param res An [crul::HttpResponse] object as returned by
#'   [crul::HttpClient()].
#' @return If no status code errors are found, nothing is returned. If status
#'   code errors are found, an error is thrown.
#' @noRd
status_catcher <- function(res) {
  if (res$status_code >= 400) {
    if (res$status_code == 401) {
      stop("Token not valid! (HTTP 401)", call. = FALSE)
    } else if (res$status_code == 404) {
      stop("No results returned for query. (HTTP 404)", call. = FALSE)
    } else if (res$status_code >= 500) {
      stop("The IUCN API is not currently available. Please try your query again
           later.", call. = FALSE)
    } else {
      res$raise_for_status()
    }
  }
}

#' Parse a JSON string to a list
#'
#' @param x (character) A JSON string.
#' @param parse (logical) Whether to parse sub-elements of the list to lists
#'   (`FALSE`) or, where possible, to data.frames (`TRUE`). Default:
#'   `TRUE`.
#'
#' @return A list.
#' @noRd
#' @importFrom jsonlite fromJSON
rl_parse <- function(x, parse) {
  fromJSON(x, parse)
}

#' Retrieve a stored API key, if needed
#'
#' @param x (character) An API key as a string. Can also be `NULL`, in which
#'   case the API key will be retrieved from the environmental variable or R
#'   option (in that order).
#'
#' @return A string. If no API key is found, an error is thrown.
#' @noRd
check_key <- function(x) {
  tmp <- if (is.null(x)) Sys.getenv("IUCN_REDLIST_KEY", "") else x
  if (tmp == "") {
    getOption("iucn_redlist_key", stop("need an API key for Red List data",
                                       call. = FALSE))
  } else {
    tmp
  }
}

#' Parse a JSON string to a list
#'
#' @return The base URL for the IUCN API
#' @noRd
rr_base <- function() "https://api.iucnredlist.org/api/v4"

#' Escape spaces in a URL string
#'
#' @param x (character) A URL string (possibly with unescaped spaces)
#' @return (character) A URL string (with any spaces escaped)
#' @noRd
space <- function(x) gsub("\\s", "%20", x)

#' Check that a value inherits the desired class
#'
#' @param x The value to be checked.
#' @param y (character) The name of a class.
#'
#' @return If the check fails, an error is thrown, otherwise, nothing is
#'   returned.
#' @noRd
assert_is <- function(x, y) {
  if (!is.null(x)) {
    if (!inherits(x, y)) {
      stop(deparse(substitute(x)), " must be of class ",
           paste0(y, collapse = ", "), call. = FALSE)
    }
  }
}

#' Check that a value has a desired length
#'
#' @param x The value to be checked.
#' @param n (numeric) The desired length.
#'
#' @return If the check fails, an error is thrown, otherwise, nothing is
#'   returned.
#' @noRd
assert_n <- function(x, n) {
  if (!is.null(x)) {
    if (!length(x) == n) {
      stop(deparse(substitute(x)), " must be length ", n, call. = FALSE)
    }
  }
}

#' Combine assessments from multiple pages of a single query
#'
#' @param res A list where each element represents the assessments from a single
#'   page of a multi-page query (such as the output of `page_assessments()`).
#' @param parse (logical) Whether to parse and combine the assessments into a
#'   data.frame (`TRUE`) or keep them as lists (`FALSE`). Default:
#'   `TRUE`.
#' @noRd
#' @return If `parse` is `TRUE`, a data.frame, otherwise, a list.
combine_assessments <- function(res, parse) {
  if (length(res) <= 1) return(rl_parse(res, parse))
  lst <- lapply(res, rl_parse, parse = parse)
  tmp <- lst[[1]]
  assessments <- lapply(lst, "[[", "assessments")
  if (parse) {
    tmp$assessments <- do.call(rbind, assessments)
  } else {
    tmp$assessments <- do.call(c, assessments)
  }
  return(tmp)
}

#' Page through assessments
#'
#' @param path (character) The full API endpoint.
#' @param key (character) An IUCN API token. See [rl_use_iucn()].
#' @param quiet (logical) Whether to suppress progress for multi-page downloads
#'   or not. Default: `FALSE` (that is, give progress). Ignored if `all =
#'   FALSE`.
#' @param ... [Curl options][curl::curl_options()] passed to the GET request via
#'   [HttpClient][crul::HttpClient()].
#'
#' @return A list with each element representing the response of one page of
#'   results.
#' @noRd
#' @importFrom cli cli_progress_bar cli_progress_update cli_progress_done
page_assessments <- function(path, key, quiet, ...) {
  out <- list()
  res <- rr_GET_raw(path, key, query = list(page = 1), ...)
  status_catcher(res)
  total_pages <- as.integer(res$response_headers$`total-pages`)
  if (length(total_pages) == 0) total_pages <- 1
  if (!quiet) cli_progress_bar(
    "Paging assessments", total = total_pages, clear = FALSE,
    format = paste0("{cli::pb_name} ({cli::pb_current}/{cli::pb_total}) | ",
                    "{cli::pb_bar} {cli::pb_percent} | ETA: {cli::pb_eta}"))
  tmp <- res$parse("UTF-8")
  if (total_pages == 1) {
    out <- tmp
  } else {
    out[[1]] <- tmp
    if (!quiet) cli_progress_update()
    for (page in 2:total_pages) {
      tmp <- rr_GET(path, key, query = list(page = page), ...)
      out[[page]] <- tmp
      if (!quiet) cli_progress_update()
    }
  }
  if (!quiet) cli_progress_done()
  return(out)
}

context("utility functions work")

test_that("ct works", {
  lst <- list(NULL, "a", NULL, "b")
  res <- ct(lst)
  expect_length(res, 2)
  expect_true(all(sapply(res, function(x) !is.null(x))))
})

test_that("rr_GET_raw works", {
  skip_on_cran()

  vcr::use_cassette("rr_GET_raw", {
    expect_no_error(res <- rr_GET_raw("taxa/family"))
  })
  expect_is(res, "HttpResponse")
  expect_equal(res$status_code, 200)
})

test_that("rr_GET works", {
  skip_on_cran()

  vcr::use_cassette("rr_GET", {
    expect_no_error(res <- rr_GET("taxa/family"))
  })
  expect_true(is.character(res))
  expect_equal(substr(res, 1, 1), "{")
})

test_that("check_key works", {
  expect_no_error(key <- check_key(NULL))
  expect_true(is.character(key))

  expect_no_error(key <- check_key("test"))
  expect_true(is.character(key))
  expect_equal(key, "test")
})

test_that("space works", {
  txt <- "Hello World"
  txt_encode <- space(txt)
  expect_true(is.character(txt_encode))
  expect_equal(txt_encode, "Hello%20World")
})

test_that("assert_is works", {
  x <- 42
  expect_no_error(assert_is(x, "numeric"))
  expect_error(assert_is(x, "character"))
  expect_no_error(assert_is(NULL, "numeric"))
})

test_that("assert_n works", {
  lst <- letters[1:5]
  expect_no_error(assert_n(lst, 5))
  expect_error(assert_n(lst, 42))
  lst2 <- as.list(lst)
  expect_no_error(assert_n(lst, 5))
  expect_error(assert_n(lst, 42))
  expect_no_error(assert_n(NULL, 5))
})

test_that("page_assessments and combine_assessments work", {
  skip_on_cran()

  vcr::use_cassette("Sturnidae_metadata", {
    res <- rr_GET_raw("taxa/family/Sturnidae",
                      key = NULL, query = list(page = 1))
  })
  total_pages <- as.integer(res$response_headers$`total-pages`)
  total_count <- as.integer(res$response_headers$`total-count`)
  vcr::use_cassette("page_assessments_multipage", {
    expect_no_error(pages <- page_assessments("taxa/family/Sturnidae",
                                              key = NULL, quiet = TRUE))
  })
  expect_true(is.list(pages))
  expect_length(pages, total_pages)
  expect_true(all(sapply(pages, function(x) is.character(x))))

  comb <- combine_assessments(pages, parse = TRUE)
  expect_true(is.list(comb))
  expect_true(is.data.frame(comb$assessments))
  expect_equal(nrow(comb$assessments), total_count)

  vcr::use_cassette("Rheidae_metadata", {
    res <- rr_GET_raw("taxa/family/Rheidae",
                      key = NULL, query = list(page = 1))
  })
  total_count <- as.integer(res$response_headers$`total-count`)

  vcr::use_cassette("page_assessments_singlepage", {
    expect_no_error(pages <- page_assessments("taxa/family/Rheidae",
                                              key = NULL, quiet = TRUE))
  })
  expect_true(is.character(pages))
  expect_length(pages, 1)

  comb <- combine_assessments(pages, parse = TRUE)
  expect_true(is.list(comb))
  expect_true(is.data.frame(comb$assessments))
  expect_equal(nrow(comb$assessments), total_count)
})

#' Get the citation for the Red List API
#'
#' Full acknowledgement and citation needs to be given for using the API. Use
#' this function to get the full citation for the current version of the Red
#' List API. More details are available here: <https://api.iucnredlist.org/>.
#'
#' @export
#' @param key (character) An IUCN API token. See [rl_use_iucn()].
#' @param ... Curl options passed to [HttpClient][crul::HttpClient()].
#' @return Red List citation as a [bibentry][utils::bibentry()] object.
#' @importFrom utils bibentry
#' @family stats
#' @examples \dontrun{
#' rl_citation()
#' }
rl_citation <- function(key = NULL, ...) {
  vers <- rl_version(key, ...)
  year <- strsplit(vers, "-")[[1]][1]
  cit <- bibentry(
    bibtype      = "Misc",
    key          = paste0("IUCN", year),
    author       = "IUCN",
    title        = "IUCN Red List of Threatened Species",
    year         = year,
    edition      = paste("Version", vers),
    howpublished = "\\url{https://www.iucnredlist.org}",
    note         = paste("Accessed on", format(Sys.Date(),"%d %B %Y")),
  )
  class(cit) <- c("citation", "bibentry")
  attr(cit, "mheader") <- "To cite the IUCN API in publications, use the
                           following citation:"
  return(cit)
}

context("rl_citation functions")

test_that("rl_citation works", {
  skip_on_cran()

  vcr::use_cassette("rl_citation", {
    aa <- rl_citation()
  })

  expect_is(aa, "bibentry")
  aa_print <- paste(capture.output(aa), collapse = " ")
  expect_match(aa_print, "IUCN Red List of Threatened Species")
  expect_match(aa_print, "www.iucnredlist.org")
})

test_that("rl_citation curl options work", {
  skip_on_cran()

  expect_error(rl_citation(timeout_ms = 1), "Timeout was reached")
})

#' Get count of species in the Red List
#'
#' Returns a count of the number of unique species which have assessments.
#'
#' @export
#' @param key (character) An IUCN API token. See [rl_use_iucn()].
#' @param ... Curl options passed to [HttpClient][crul::HttpClient()]
#' @return An integer representing the number of unique species represented
#'   within the IUCN database.
#' @references API docs at <https://api.iucnredlist.org/>.
#' @family stats
#' @examples \dontrun{
#' # Get count of species with assessments
#' rl_sp_count()
#' }
rl_sp_count <- function(key = NULL, ...) {
  rl_parse(rl_sp_count_(key, ...), TRUE)$count
}

#' @export
#' @rdname rl_sp_count
rl_sp_count_ <- function(key = NULL, ...) {
  assert_is(key, "character")
  rr_GET("statistics/count", key, ...)
}

context("rl_sp_count functions")

test_that("high level functions", {
  skip_on_cran()

  vcr::use_cassette("rl_sp_count", {
    aa <- rl_sp_count()
  })
  expect_is(aa, "integer")

})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_sp_count_", {
    aa <- rl_sp_count_()
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_is(aajson$count, "integer")
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_sp_count(key = 5), "key must be of class character")
  expect_error(rl_sp_count(key = matrix()), "key must be of class character")
})

# Copied from ggplot2

# By default, if vdiffr is not installed, all visual tests are skipped unless
# VDIFFR_RUN_TESTS is explicitly set to "true", which should be the case only on
# a GitHub Actions CI runner with stable version of R.

if (requireNamespace("vdiffr", quietly = TRUE) &&
    utils::packageVersion("testthat") >= "3.0.3") {
  expect_doppelganger <- vdiffr::expect_doppelganger
} else {
  # If vdiffr is not available and visual tests are explicitly required,
  # raise error.
  if (identical(Sys.getenv("VDIFFR_RUN_TESTS"), "true")) {
    abort("vdiffr is not installed")
  }

  # Otherwise, assign a dummy function
  expect_doppelganger <- function(...) skip("vdiffr is not installed.")
}

#' Conservation actions needed assessment summary
#'
#' Return the latest assessments with a given conservation action needed (e.g.,
#' Land/water management or Species recovery). These conservation action codes
#' correspond to the IUCN Red List Conservation Actions Classification Scheme
#' (v2.0).
#'
#' @export
#' @param code (character) The code of the conservation action to look up. If
#'   not supplied, a list of all conservation actions will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family conservation
#' @examples \dontrun{
#' # Get list of all conservation actions
#' rl_actions()
#' # Get assessments with Species recovery conservation action needed
#' rl_actions("3_2")
#' }
rl_actions <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                       page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_actions_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_actions
rl_actions_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                        quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("conservation_actions", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Research needed assessment summary
#'
#' Return the latest assessments with a given research needed (e.g., Taxonomy or
#' Population trends). These research codes correspond to the IUCN Red List
#' Research Needed Classification Scheme (v1.0).
#'
#' @export
#' @param code (character) The code of the research type to look up. If not
#'   supplied, a list of all research types will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family conservation
#' @examples \dontrun{
#' # Get list of all research types
#' rl_research()
#' # Get assessments with Taxonomy research needed
#' rl_research("1_1")
#' }
rl_research <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                        page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_research_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_research
rl_research_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                         quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("research", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

context("rl_research functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_research-def", {
    aa <- rl_research()
  })

  expect_is(aa, "list")
  expect_named(aa, c("research"))
  expect_is(aa$research, "data.frame")

  vcr::use_cassette("rl_research", {
    aa <- rl_research("4")
  })

  expect_is(aa, "list")
  expect_named(aa, c("research", "assessments", "filters"))
  expect_is(aa$research, "list")
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_research-def-not-parsing", {
    aa <- rl_research(parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("research"))
  expect_is(aa$research, "list")

  vcr::use_cassette("rl_research-not-parsing", {
    aa <- rl_research("4", parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("research", "assessments", "filters"))
  expect_is(aa$research, "list")
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_research_-def", {
    aa <- rl_research_()
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("research"))
  expect_is(aajson$research, "data.frame")

  vcr::use_cassette("rl_research_", {
    aa <- rl_research_("4", all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("research", "assessments", "filters"))
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_research(5), "code must be of class character")
  expect_error(rl_research(list()), "code must be of class character")

  expect_error(rl_research(key = 5), "key must be of class character")
  expect_error(rl_research(key = matrix()), "key must be of class character")

  expect_error(rl_research(parse = 5), "parse must be of class logical")
  expect_error(rl_research(parse = matrix()), "parse must be of class logical")

  expect_error(rl_research(page = "next"), "page must be of class integer")
  expect_error(rl_research(all = "yes"), "all must be of class logical")
  expect_error(rl_research(quiet = "no"), "quiet must be of class logical")

  # lengths
  expect_error(rl_research(page = 1:2), "page must be length 1")

})

#' Comprehensive group assessment summary
#'
#' Returns a list of the latest assessments for a comprehensive group name.
#'
#' @export
#' @param name (character) The code of the comprehensive group to look up. If
#'   not supplied, a list of all comprehensive groups will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family groups
#' @examples \dontrun{
#' # Get list of all comprehensive groups
#' rl_comp_groups()
#' # Get assessment summary for sea snakes
#' rl_comp_groups('seasnakes')
#' }
rl_comp_groups <- function(name = NULL, key = NULL, parse = TRUE, all = TRUE,
                           page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_comp_groups_(name, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_comp_groups
rl_comp_groups_ <- function(name = NULL, key = NULL, all = TRUE, page = 1,
                            quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(name, "character")
  assert_n(name, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("comprehensive_groups", name, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Get common names for a given taxonomic name
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of
#'   [rl_species_latest()]`$taxon$common_names` as of **rredlist** version
#'   1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_common_names <- function(name = NULL, key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_common_names()",
    details = "Please use `rl_species_latest()$taxon$common_names` instead")
}

#' @export
#' @rdname rl_common_names
#' @usage NULL
rl_common_names_ <- function(name = NULL, key = NULL, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_common_names_()",
                            "rl_species_latest_()")
}

#' Get historical assessments by taxon name, IUCN id, and region
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of [rl_species()] as of
#'   **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_history <- function(name = NULL, id = NULL, region = NULL,
                       key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_history()", "rl_species()")
}

#' @export
#' @rdname rl_history
#' @usage NULL
rl_history_ <- function(name = NULL, id = NULL, region = NULL,
                        key = NULL, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_history()", "rl_species_()")
}

#' Get species conservation measures by taxon name, IUCN id, and region
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of
#'   [rl_species_latest()]`$conservation_actions` as of **rredlist** version
#'   1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_measures <- function(name = NULL, id = NULL, region = NULL,
                        key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_measures()",
    details = "Please use `rl_species_latest()$conservation_actions` instead")
}

#' @export
#' @rdname rl_measures
#' @usage NULL
rl_measures_ <- function(name = NULL, id = NULL, region = NULL,
                         key = NULL, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_measures_()", "rl_species_latest_()")
}

#' Get species narrative information by taxon name, IUCN id, and region
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of
#'   [rl_species_latest()]`$documentation` as of **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_narrative <- function(name = NULL, id = NULL, region = NULL,
                         key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_narrative()",
    details = "Please use `rl_species_latest()$documentation` instead")
}

#' @export
#' @rdname rl_narrative
#' @usage NULL
rl_narrative_ <- function(name = NULL, id = NULL, region = NULL,
                          key = NULL, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_narrative_()", "rl_species_latest_()")
}

#' Get country occurrence by species name or ID
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of
#'   [rl_species_latest()]`$locations` as of **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_occ_country <- function(name = NULL, id = NULL, region = NULL,
                           key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_occ_country()",
    details = "Please use `rl_species_latest()$locations` instead")
}

#' @export
#' @rdname rl_occ_country
#' @usage NULL
rl_occ_country_ <- function(name = NULL, id = NULL, region = NULL,
                            key = NULL, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_occ_country_()",
                            "rl_species_latest_()")
}

#' Get regions
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of
#'   [rl_scopes(code = NULL)][rl_scopes()] as of **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_regions <- function(key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_regions()",
    details = "Please use `rl_scopes(code = NULL)` instead")
}

#' @export
#' @rdname rl_regions
#' @usage NULL
rl_regions_ <- function(key = NULL, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_regions_()",
    details = "Please use `rl_scopes_(code = NULL)` instead")
}

#' Search by taxon name, IUCN id, and region
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of [rl_species()], [rl_sis()],
#'   and [rl_scopes()] as of **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_search <- function(name = NULL, id = NULL, region = NULL,
                      key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_search()",
    details = "Please use `rl_species()`, `rl_sis()`, or `rl_scopes()` instead")
}

#' @export
#' @rdname rl_search
#' @usage NULL
rl_search_ <- function(name = NULL, id = NULL, region = NULL,
                       key = NULL, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_search_()",
    details = paste("Please use `rl_species_()`, `rl_sis_()`,",
                    "or `rl_scopes_()` instead"))
}

#' Get species
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of [rl_kingdom()],
#'   [rl_phylum()], [rl_class()], [rl_order()], and [rl_family()] as of
#'   **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_sp <- function(page = 0, key = NULL, parse = TRUE, all = FALSE,
                  quiet = FALSE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_sp()",
    details = paste("Please use `rl_kingdom()`, `rl_phylum()`,",
                    "`rl_class()`, `rl_order()`, or `rl_family()` instead"))
}

#' @export
#' @rdname rl_sp
#' @usage NULL
rl_sp_ <- function(page, key = NULL, all = FALSE, quiet = FALSE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_sp_()",
    details = paste("Please use `rl_kingdom_()`, `rl_phylum_()`,",
                    "`rl_class_()`, `rl_order_()`, or `rl_family_()` instead"))
}

#' Get species by category
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of [rl_categories()] as of
#'   **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_sp_category <- function(category, key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_sp_category()", "rl_categories()")
}

#' @export
#' @rdname rl_sp_category
#' @usage NULL
rl_sp_category_ <- function(category, key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_sp_category_()", "rl_categories_()")
}

#' Get citations by taxon name, IUCN id, and region
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of
#'   [rl_species_latest()]`$citation` as of **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_sp_citation <- function(name = NULL, id = NULL, region = NULL,
                           key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_sp_citation()",
    details = "Please use `rl_species_latest()$citation` instead")
}

#' @export
#' @rdname rl_sp_citation
#' @usage NULL
rl_sp_citation_ <- function(name = NULL, id = NULL, region = NULL,
                            key = NULL, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_sp_citation_()",
                            "rl_species_latest_()")
}

#' Get species by country
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of [rl_countries()] as of
#'   **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_sp_country <- function(country, key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_sp_country()", "rl_countries()")
}

#' @export
#' @rdname rl_sp_country
#' @usage NULL
rl_sp_country_ <- function(country, key = NULL, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_sp_country_()", "rl_countries_()")
}

#' Get species synonym information by taxonomic name
#'
#' @description `r lifecycle::badge('deprecated')`
#'
#'   This function is fully deprecated in favor of
#'   [rl_species_latest()]`$taxon$synonyms` as of **rredlist** version 1.0.0.
#' @keywords internal
#' @export
#' @usage NULL
rl_synonyms <- function(name = NULL, key = NULL, parse = TRUE, ...) {
  lifecycle::deprecate_stop(
    "1.0.0", "rl_synonyms()",
    details = "Please use `rl_species_latest()$taxon$synonyms` instead")
}

#' @export
#' @rdname rl_synonyms
#' @usage NULL
rl_synonyms_ <- function(name = NULL, key = NULL, ...) {
  lifecycle::deprecate_stop("1.0.0", "rl_synonyms_()",
                            "rl_species_latest()")
}

test_that("defunct functions works", {
  lifecycle::expect_defunct(rl_common_names())
  lifecycle::expect_defunct(rl_common_names_())
  lifecycle::expect_defunct(rl_history())
  lifecycle::expect_defunct(rl_history_())
  lifecycle::expect_defunct(rl_measures())
  lifecycle::expect_defunct(rl_measures_())
  lifecycle::expect_defunct(rl_narrative())
  lifecycle::expect_defunct(rl_narrative_())
  lifecycle::expect_defunct(rl_occ_country())
  lifecycle::expect_defunct(rl_occ_country_())
  lifecycle::expect_defunct(rl_regions())
  lifecycle::expect_defunct(rl_regions_())
  lifecycle::expect_defunct(rl_search())
  lifecycle::expect_defunct(rl_search_())
  lifecycle::expect_defunct(rl_sp())
  lifecycle::expect_defunct(rl_sp_())
  lifecycle::expect_defunct(rl_sp_category())
  lifecycle::expect_defunct(rl_sp_category_())
  lifecycle::expect_defunct(rl_sp_citation())
  lifecycle::expect_defunct(rl_sp_citation_())
  lifecycle::expect_defunct(rl_sp_country())
  lifecycle::expect_defunct(rl_sp_country_())
  lifecycle::expect_defunct(rl_synonyms())
  lifecycle::expect_defunct(rl_synonyms_())
})

#' Red List category assessment summary
#'
#' Return a list of the latest assessments for a given category (e.g., Not
#' Evaluated, Data Deficient, Least Concern, Near Threatened, Vulnerable,
#' Endangered, Critically Endangered, Extinct in the Wild and Extinct). This
#' endpoint returns categories for the current IUCN Red List Categories and
#' Criteria (v3.1) as well as older versions (i.e., v2.3). Note that a code may
#' not be unique across Categories and Criteria versions. Therefore, codes like
#' "EX" will return assessments for EX v3.1 and EX v2.3.
#'
#' @export
#' @param code (character) The code of the Red List category to look up. If not
#'   supplied, a list of all categories will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family threats
#' @examples \dontrun{
#' # Get all Red List categories
#' rl_categories()
#' # Get assessments for the Vulnerable category
#' rl_categories("VU")
#' }
rl_categories <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                          page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_categories_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_categories
rl_categories_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                           quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("red_list_categories", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' IUCN Red List color scales
#'
#' ggplot2 color scales using the colors for the IUCN Red List threat categories, as
#' documented in [this
#' chart](https://nc.iucnredlist.org/redlist/resources/files/1646067752-FINAL_IUCN_Red_List_colour_chart.pdf).
#'
#' @param aesthetics Character string or vector of character strings listing the
#'   name(s) of the aesthetic(s) that this scale works with. This can be useful,
#'   for example, to apply color settings to the `color` and `fill` aesthetics
#'   at the same time, via `aesthetics = c("color", "fill")`.
#' @param ... Arguments passed on to [ggplot2::discrete_scale] (except for
#'   `na.value` which is already set according to the IUCN standard of
#'   `#C1B5A5`)
#' @rdname scale_iucn
#' @export
#' @examplesIf require(ggplot2)
#' library(ggplot2)
#' categories <- c("NE", "DD", "LC", "NT", "VU", "EN", "CR", "RE", "EW", "EX")
#' df <- data.frame(
#'   x = runif(1000, 0, 10), y = runif(1000, 0, 10),
#'   color = sample(categories, 1000, TRUE), shape = 21
#' )
#' ggplot(df) +
#'   geom_point(aes(x = x, y = y, fill = color), shape = 21) +
#'   scale_fill_iucn(name = "IUCN Category") +
#'   theme_classic()
scale_color_iucn <- function(...) {
  scale_discrete_iucn("color", ...)
}

#' @export
#' @rdname scale_iucn
#' @usage NULL
scale_colour_iucn <- scale_color_iucn

#' @rdname scale_iucn
#' @export
scale_fill_iucn <- function(...) {
  scale_discrete_iucn("fill", ...)
}

#' @export
#' @rdname scale_iucn
#' @importFrom rlang check_installed
scale_discrete_iucn <- function(aesthetics, ...) {
  check_installed("ggplot2", reason = "to use `scale_discrete_iucn()`")
  values <- c(
    "NE" = "#FFFFFF",
    "DD" = "#D1D1C6",
    "LC" = "#60C659",
    "NT" = "#CCE226",
    "VU" = "#F9E814",
    "EN" = "#FC7F3F",
    "CR" = "#D81E05",
    "RE" = "#9B4F96",
    "EW" = "#542344",
    "EX" = "#000000"
  )
  pal <- function(n) {
    if (n > length(values)) {
      cli::cli_abort("Insufficient values in manual scale. {n} needed but only
                     {length(values)} provided.")
    }
    values
  }

  ggplot2::discrete_scale(aesthetics, palette = pal, breaks = names(values),
                          na.value = "#C1B5A5", ...)
}

#' Green Status assessment summary
#'
#' List all Green Status assessments.
#'
#' @export
#' @template all
#' @template curl
#' @template info
#' @family groups
#' @examples \dontrun{
#' # Get list of Green Status assessments
#' rl_green()
#' }
rl_green <- function(key = NULL, parse = TRUE, ...) {
  assert_is(parse, "logical")

  rl_parse(rl_green_(key, ...), parse)
}

#' @export
#' @rdname rl_green
rl_green_ <- function(key = NULL, ...) {
  assert_is(key, "character")

  rr_GET("green_status/all", key, ...)
}

context("rl_green functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_green", {
    aa <- rl_green()
  })

  expect_is(aa, "list")
  expect_named(aa, c("assessments"))
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_green-not-parsing", {
    aa <- rl_green(parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("assessments"))
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_green_", {
    aa <- rl_green_()
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("assessments"))
  expect_is(aajson$assessments, "data.frame")
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_green(key = 5), "key must be of class character")
  expect_error(rl_green(key = matrix()), "key must be of class character")

  expect_error(rl_green(parse = 5), "parse must be of class logical")
  expect_error(rl_green(parse = matrix()), "parse must be of class logical")
})

context("rl_categories functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_categories-def", {
    aa <- rl_categories()
  })

  expect_is(aa, "list")
  expect_named(aa, c("red_list_categories"))
  expect_is(aa$red_list_categories, "data.frame")

  vcr::use_cassette("rl_categories", {
    aa <- rl_categories("EW")
  })

  expect_is(aa, "list")
  expect_named(aa, c("red_list_category", "assessments", "filters"))
  expect_is(aa$red_list_category, "data.frame")
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_categories-def-not-parsing", {
    aa <- rl_categories(parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("red_list_categories"))
  expect_is(aa$red_list_categories, "list")

  vcr::use_cassette("rl_categories-not-parsing", {
    aa <- rl_categories("EW", parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("red_list_category", "assessments", "filters"))
  expect_is(aa$red_list_category, "list")
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_categories_-def", {
    aa <- rl_categories_()
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("red_list_categories"))
  expect_is(aajson$red_list_categories, "data.frame")

  vcr::use_cassette("rl_categories_", {
    aa <- rl_categories_("EW", all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("red_list_category", "assessments", "filters"))
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_categories(5), "code must be of class character")
  expect_error(rl_categories(list()), "code must be of class character")

  expect_error(rl_categories(key = 5), "key must be of class character")
  expect_error(rl_categories(key = matrix()), "key must be of class character")

  expect_error(rl_categories(parse = 5), "parse must be of class logical")
  expect_error(rl_categories(parse = matrix()),
               "parse must be of class logical")

  expect_error(rl_categories(page = "next"), "page must be of class integer")
  expect_error(rl_categories(all = "yes"), "all must be of class logical")
  expect_error(rl_categories(quiet = "no"), "quiet must be of class logical")

  # lengths
  expect_error(rl_categories(page = 1:2), "page must be length 1")
})

suppressPackageStartupMessages(library(ggplot2, quietly = TRUE))

categories <- c("NE", "DD", "LC", "NT", "VU", "EN", "CR", "RE", "EW", "EX")
set.seed(1234)
df <- data.frame(
  x = runif(1000, 0, 10), y = runif(1000, 0, 10),
  color = sample(categories, 1000, TRUE)
)

test_that("scale_fill_iucn works", {
  gg <- ggplot(df) +
    geom_point(aes(x = x, y = y, fill = color), shape = 21) +
    scale_fill_iucn() +
    theme_classic()
  expect_doppelganger("scale_fill_iucn", gg)
})

test_that("scale_color_iucn works", {
  gg <- ggplot(df) +
    geom_point(aes(x = x, y = y, color = color), shape = 21) +
    scale_color_iucn() +
    theme_classic()
  expect_doppelganger("scale_color_iucn", gg)
})

# nocov start

release_questions <- function() {
  c(
    "Did you update the CITATION file?",
    "Did you update and knit the README?",
    "Did you rebuild the codemeta.json file using codemetar?",
    "Did you rebuild the vignette(s) using `vignettes/precompile.R`?"
  )
}

# nocov end

list(
  rd_family_title = list(taxa = "Assessments by taxa:",
                         stats = "Red List information and statistics:",
                         habitat = "Assessments by habitats and systems:",
                         geo = "Assessments by geographic regions:",
                         conservation = "Assessments by conservation actions or research needed:",
                         threats = "Assessments by threat details:",
                         groups = "Assessments for specific sets of taxa")
)

#' @keywords internal
"_PACKAGE"

#' @title rredlist
#' @section Authentication:
#' IUCN requires you to get your own API key, an alphanumeric string that you
#' need to send in every request. See [rl_use_iucn()] for help getting and
#' storing it. Get it at <https://api.iucnredlist.org/users/sign_up>. Keep this
#' key private. You can pass the key in to each function via the `key`
#' parameter, but it's better to store the key either as an environment variable
#' (`IUCN_REDLIST_KEY`) or an R option (`iucn_redlist_key`) - we recommend using
#' the former option.
#'
#' @section High vs. Low level package APIs:
#' **High level API**
#' High level functions do the HTTP request and parse data to a data.frame for
#' ease of downstream use. The high level functions have no underscore on
#' the end of the function name, e.g., [rl_species()].
#'
#' **Low level API**
#' The parsing to data.frame in the high level API does take extra time.
#' The low level API only does the HTTP request, and gives back JSON without
#' doing any more parsing. The low level functions DO have an underscore on
#' the end of the function name, e.g., [rl_species_()].
#'
#' @section No Spatial:
#' This package does not include support for spatial data from the IUCN Red
#' List. IUCN Red List Spatial data can be downloaded at
#' <https://www.iucnredlist.org/resources/spatial-data-download>.
#'
#' @section Citing the IUCN Red List API:
#' Get the proper citation for the version of the Red List you are using
#' by programatically running [rl_citation()]. Also see
#' <https://api.iucnredlist.org/>.
#'
#' @section Red List API Terms of Use:
#' See <https://www.iucnredlist.org/terms/terms-of-use>.
#'
#' @section Rate limiting:
#' From the IUCN folks: Too many frequent calls, or too many calls per day
#' might get your access blocked temporarily. If you're a heavy API user, the
#' Red List Unit asked that you contact them, as there might be better options.
#' They suggest a 2-second delay between your calls if you plan to make a
#' lot of calls.
#'
#' @importFrom jsonlite fromJSON
#' @importFrom lifecycle deprecated
#' @name rredlist-package
#' @aliases rredlist
#' @keywords package
NULL

#' Threat assessment summary
#'
#' Return a list of the latest assessments which are subject to a specific
#' threat (e.g., energy production and mining, climate change, and severe
#' weather). This will only return assessments for the threat code specified.
#' You will need to do additional requests for sub-threats (e.g., a request for
#' threat code 2_1 will need additional requests for codes 2_1_1, 2_1_2, etc.).
#'
#' @export
#' @param code (character) The code of the threat to look up. If not supplied, a
#'   list of all threats will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family threats
#' @examples \dontrun{
#' # Get all threats
#' rl_threats()
#' # Get assessment summary for intentional hunting and trapping
#' rl_threats("5_1_1")
#' }
rl_threats <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                       page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_threats_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_threats
rl_threats_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                        quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("threats", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Stress assessment summary
#'
#' Return a list of the latest assessments based on the stresses species are
#' subject to (e.g., Ecosystem degradation, species disturbance, etc.).
#'
#' @export
#' @param code (character) The code of the stress to look up. If not supplied, a
#'   list of all stresses will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family threats
#' @examples \dontrun{
#' # Get all stresses
#' rl_stresses()
#' # Get assessment summary for ecosystem degradation stress
#' rl_stresses("1_2")
#' }
rl_stresses <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                        page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_stresses_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_stresses
rl_stresses_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                         quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("stresses", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Use and trade assessment summary
#'
#' Return a list of the latest assessments which are subject to a specific use
#' and trade.
#'
#' @export
#' @param code (character) The code of the use and trade to look up. If not
#'   supplied, a list of all uses and trades will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family threats
#' @examples \dontrun{
#' # Get all stresses
#' rl_use_and_trade()
#' # Get assessment summary for medicinal use and trade
#' rl_use_and_trade("3")
#' }
rl_use_and_trade <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                             page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_use_and_trade_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_use_and_trade
rl_use_and_trade_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                              quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("use_and_trade", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

context("rl_use_and_trade functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_use_and_trade-def", {
    aa <- rl_use_and_trade()
  })

  expect_is(aa, "list")
  expect_named(aa, c("use_and_trade"))
  expect_is(aa$use_and_trade, "data.frame")

  vcr::use_cassette("rl_use_and_trade", {
    aa <- rl_use_and_trade("4")
  })

  expect_is(aa, "list")
  expect_named(aa, c("use_and_trade", "assessments", "filters"))
  expect_is(aa$use_and_trade, "list")
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_use_and_trade-def-not-parsing", {
    aa <- rl_use_and_trade(parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("use_and_trade"))
  expect_is(aa$use_and_trade, "list")

  vcr::use_cassette("rl_use_and_trade-not-parsing", {
    aa <- rl_use_and_trade("4", parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("use_and_trade", "assessments", "filters"))
  expect_is(aa$use_and_trade, "list")
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_use_and_trade_-def", {
    aa <- rl_use_and_trade_()
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("use_and_trade"))
  expect_is(aajson$use_and_trade, "data.frame")

  vcr::use_cassette("rl_use_and_trade_", {
    aa <- rl_use_and_trade_("4", all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("use_and_trade", "assessments", "filters"))
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_use_and_trade(5), "code must be of class character")
  expect_error(rl_use_and_trade(list()), "code must be of class character")

  expect_error(rl_use_and_trade(key = 5), "key must be of class character")
  expect_error(rl_use_and_trade(key = matrix()),
               "key must be of class character")

  expect_error(rl_use_and_trade(parse = 5), "parse must be of class logical")
  expect_error(rl_use_and_trade(parse = matrix()),
               "parse must be of class logical")

  expect_error(rl_use_and_trade(page = "next"), "page must be of class integer")
  expect_error(rl_use_and_trade(all = "yes"), "all must be of class logical")
  expect_error(rl_use_and_trade(quiet = "no"), "quiet must be of class logical")

  # lengths
  expect_error(rl_use_and_trade(page = 1:2), "page must be length 1")
})

context("rl_threats functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_threats-def", {
    aa <- rl_threats()
  })

  expect_is(aa, "list")
  expect_named(aa, c("threats"))
  expect_is(aa$threats, "data.frame")

  vcr::use_cassette("rl_threats", {
    aa <- rl_threats("9_5_1")
  })

  expect_is(aa, "list")
  expect_named(aa, c("threat", "assessments", "filters"))
  expect_is(aa$threat, "list")
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_threats-def-not-parsing", {
    aa <- rl_threats(parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("threats"))
  expect_is(aa$threats, "list")

  vcr::use_cassette("rl_threats-not-parsing", {
    aa <- rl_threats("9_5_1", parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("threat", "assessments", "filters"))
  expect_is(aa$threat, "list")
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_threats_-def", {
    aa <- rl_threats_()
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("threats"))
  expect_is(aajson$threats, "data.frame")

  vcr::use_cassette("rl_threats_", {
    aa <- rl_threats_("9_5_1", all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("threat", "assessments", "filters"))
})

test_that("secret query parameters work", {
  skip_on_cran()

  vcr::use_cassette("rl_threats_no_args", {
    aa <- rl_threats("9_5_1")
  })
  vcr::use_cassette("rl_threats_with_args", {
    bb <- rl_threats("9_5_1", latest = TRUE, year_published = 2017,
                     scope_code = 2)
  })

  aa_filt <- subset(aa$assessments, latest & year_published == "2017" &
                      unlist(lapply(aa$assessments$scopes,
                                    function(df) "2" %in% df$code)))
  expect_equal(nrow(aa_filt), nrow(bb$assessments))
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_threats(5), "code must be of class character")
  expect_error(rl_threats(list()), "code must be of class character")

  expect_error(rl_threats(key = 5), "key must be of class character")
  expect_error(rl_threats(key = matrix()), "key must be of class character")

  expect_error(rl_threats(parse = 5), "parse must be of class logical")
  expect_error(rl_threats(parse = matrix()), "parse must be of class logical")

  expect_error(rl_threats(page = "next"), "page must be of class integer")
  expect_error(rl_threats(all = "yes"), "all must be of class logical")
  expect_error(rl_threats(quiet = "no"), "quiet must be of class logical")

  # lengths
  expect_error(rl_threats(page = 1:2), "page must be length 1")

})

#' Population trend assessment summary
#'
#' Return a list of the latest assessments based on a population trend (i.e.
#' increasing, decreasing, stable or unknown).
#'
#' @export
#' @param code (character) The code of the growth form to look up. If not
#'   supplied, a list of all growth forms will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family threats
#' @examples \dontrun{
#' # Get list of all population trends
#' rl_pop_trends()
#' # Get assessment summary for stable population trends
#' rl_pop_trends("2")
#' }
rl_pop_trends <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                          page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_pop_trends_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_pop_trends
rl_pop_trends_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                           quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("population_trends", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' Country assessment summary
#'
#' Returns a collection of assessments for a given country code.
#'
#' @export
#' @param code (character) The ISO alpha-2 code of the country to look up. If
#'   not supplied, a list of all countries will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family geo
#' @examples \dontrun{
#' # Get list of all countries
#' rl_countries()
#' # Get assessments for Madagascar
#' rl_countries("MG")
#' }
rl_countries <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                         page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_countries_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_countries
rl_countries_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                          quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("countries", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

context("rl_countries functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_countries-def", {
    aa <- rl_countries()
  })

  expect_is(aa, "list")
  expect_named(aa, c("countries"))
  expect_is(aa$countries, "data.frame")

  vcr::use_cassette("rl_countries", {
    aa <- rl_countries("VA")
  })

  expect_is(aa, "list")
  expect_named(aa, c("country", "assessments", "filters"))
  expect_is(aa$country, "list")
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_countries-def-not-parsing", {
    aa <- rl_countries(parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("countries"))
  expect_is(aa$countries, "list")

  vcr::use_cassette("rl_countries-not-parsing", {
    aa <- rl_countries("VA", parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("country", "assessments", "filters"))
  expect_is(aa$country, "list")
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_countries_-def", {
    aa <- rl_countries_()
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("countries"))
  expect_is(aajson$countries, "data.frame")

  vcr::use_cassette("rl_countries_", {
    aa <- rl_countries_("VA", all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("country", "assessments", "filters"))
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_countries(5), "code must be of class character")
  expect_error(rl_countries(list()), "code must be of class character")

  expect_error(rl_countries(key = 5), "key must be of class character")
  expect_error(rl_countries(key = matrix()), "key must be of class character")

  expect_error(rl_countries(parse = 5), "parse must be of class logical")
  expect_error(rl_countries(parse = matrix()), "parse must be of class logical")

  expect_error(rl_countries(page = "next"), "page must be of class integer")
  expect_error(rl_countries(all = "yes"), "all must be of class logical")
  expect_error(rl_countries(quiet = "no"), "quiet must be of class logical")

  # lengths
  expect_error(rl_countries(page = 1:2), "page must be length 1")

})

#' Habitat assessment summary
#'
#' Return the latest assessments for a given habitat (e.g., Forest - Temperate
#' or Marine Intertidal). These habitat codes correspond to the IUCN Red List
#' Habitats Classification Scheme (v3.1).
#'
#' @export
#' @param code (character) The code of the habitat to look up. If not supplied,
#'   a list of all habitats will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family habitat
#' @examples \dontrun{
#' # Get list of all habitats
#' rl_habitats()
#' # Get assessments for Marine Intertidal habitat
#' rl_habitats("12")
#' }
rl_habitats <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                        page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_habitats_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_habitats
rl_habitats_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                         quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("habitats", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

#' System assessment summary
#'
#' Return the latest assessments for a given system (e.g., terrestrial,
#' freshwater or marine).
#'
#' @export
#' @param code (character) The code of the system to look up. If not supplied, a
#'   list of all systems will be returned.
#' @template all
#' @template filters
#' @template info
#' @template page
#' @family habitat
#' @examples \dontrun{
#' # Get list of all systems
#' rl_systems()
#' # Get assessment summary for marine system
#' rl_systems("2")
#' }
rl_systems <- function(code = NULL, key = NULL, parse = TRUE, all = TRUE,
                       page = 1, quiet = FALSE, ...) {
  assert_is(parse, "logical")
  assert_is(all, "logical")

  res <- rl_systems_(code, key, all, page, quiet, ...)
  if (all) {
    combine_assessments(res, parse)
  } else {
    rl_parse(res, parse)
  }
}

#' @export
#' @rdname rl_systems
rl_systems_ <- function(code = NULL, key = NULL, all = TRUE, page = 1,
                        quiet = FALSE, ...) {
  assert_is(key, "character")
  assert_is(code, "character")
  assert_n(code, 1)
  assert_is(page, c("integer", "numeric"))
  assert_n(page, 1)
  assert_is(all, "logical")
  assert_is(quiet, "logical")

  path <- paste("systems", code, sep = "/")

  if (all) {
    page_assessments(path, key, quiet, ...)
  } else {
    rr_GET(path, key, query = list(page = page), ...)
  }
}

context("rl_habitats functions")

test_that("high level works - parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_habitats-def", {
    aa <- rl_habitats()
  })

  expect_is(aa, "list")
  expect_named(aa, c("habitats"))
  expect_is(aa$habitats, "data.frame")

  vcr::use_cassette("rl_habitats", {
    aa <- rl_habitats("2")
  })

  expect_is(aa, "list")
  expect_named(aa, c("habitat", "assessments", "filters"))
  expect_is(aa$habitat, "list")
  expect_is(aa$assessments, "data.frame")
})

test_that("high level works - not parsing", {
  skip_on_cran()

  vcr::use_cassette("rl_habitats-def-not-parsing", {
    aa <- rl_habitats(parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("habitats"))
  expect_is(aa$habitats, "list")

  vcr::use_cassette("rl_habitats-not-parsing", {
    aa <- rl_habitats("2", parse = FALSE)
  })

  expect_is(aa, "list")
  expect_named(aa, c("habitat", "assessments", "filters"))
  expect_is(aa$habitat, "list")
  expect_is(aa$assessments, "list")
})

test_that("low level works", {
  skip_on_cran()

  vcr::use_cassette("rl_habitats_-def", {
    aa <- rl_habitats_()
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("habitats"))
  expect_is(aajson$habitats, "data.frame")

  vcr::use_cassette("rl_habitats_", {
    aa <- rl_habitats_("2", all = FALSE)
  })

  aajson <- jsonlite::fromJSON(aa)

  expect_is(aa, "character")
  expect_is(aajson, "list")
  expect_named(aajson, c("habitat", "assessments", "filters"))
})

test_that("fails well", {
  skip_on_cran()

  expect_error(rl_habitats(5), "code must be of class character")
  expect_error(rl_habitats(list()), "code must be of class character")

  expect_error(rl_habitats(key = 5), "key must be of class character")
  expect_error(rl_habitats(key = matrix()), "key must be of class character")

  expect_error(rl_habitats(parse = 5), "parse must be of class logical")
  expect_error(rl_habitats(parse = matrix()), "parse must be of class logical")

  expect_error(rl_habitats(page = "next"), "page must be of class integer")
  expect_error(rl_habitats(all = "yes"), "all must be of class logical")
  expect_error(rl_habitats(quiet = "no"), "quiet must be of class logical")

  # lengths
  expect_error(rl_habitats(page = 1:2), "page must be length 1")

})

context("rl_taxa functions")

ranks <- c("kingdom", "phylum", "class", "order", "family")
names <- c("Chromista", "Porifera", "Hydrozoa", "Crocodylia", "Hominidae")

for (i in seq_along(ranks)) {
  rank <- ranks[i]
  name <- names[i]
  fn_name <- paste0("rl_", rank)
  fn_name_ <- paste0("rl_", rank, "_")
  fn <- get(fn_name)
  fn_ <- get(fn_name_)
  test_that("high level works - parsing", {
    skip_on_cran()

    vcr::use_cassette(paste0(fn_name, "-def"), {
      aa <- fn()
    })

    expect_is(aa, "list")
    expect_named(aa, c(paste0(rank, "_names")))
    expect_is(aa[[paste0(rank, "_names")]], "character")

    vcr::use_cassette(fn_name, {
      aa <- fn(name)
    })

    expect_is(aa, "list")
    expect_named(aa, c("assessments", "filters"))
    expect_is(aa$filters, "list")
    expect_is(aa$filters[[paste0(rank, "_name")]], "character")
    expect_equal(aa$filters[[paste0(rank, "_name")]], name)
    expect_is(aa$assessments, "data.frame")
  })

  test_that("high level works - not parsing", {
    skip_on_cran()

    vcr::use_cassette(paste0(fn_name, "-def-not-parsing"), {
      aa <- fn(parse = FALSE)
    })

    expect_is(aa, "list")
    expect_named(aa, c(paste0(rank, "_names")))
    expect_is(aa[[paste0(rank, "_names")]], "list")

    vcr::use_cassette(paste0(fn_name, "-not-parsing"), {
      aa <- fn(name, parse = FALSE)
    })

    expect_named(aa, c("assessments", "filters"))
    expect_is(aa$filters, "list")
    expect_is(aa$filters[[paste0(rank, "_name")]], "character")
    expect_equal(aa$filters[[paste0(rank, "_name")]], name)
    expect_is(aa$assessments, "list")
  })

  test_that("low level works", {
    skip_on_cran()

    vcr::use_cassette(paste0(fn_name_, "-def"), {
      aa <- fn_()
    })

    aajson <- jsonlite::fromJSON(aa)

    expect_is(aa, "character")
    expect_is(aajson, "list")
    expect_named(aajson, c(paste0(rank, "_names")))
    expect_is(aajson[[paste0(rank, "_names")]], "character")

    vcr::use_cassette(fn_name_, {
      aa <- fn_(name, all = FALSE)
    })

    aajson <- jsonlite::fromJSON(aa)

    expect_is(aa, "character")
    expect_named(aajson, c("assessments", "filters"))
    expect_is(aajson$filters, "list")
    expect_is(aajson$filters[[paste0(rank, "_name")]], "character")
    expect_equal(aajson$filters[[paste0(rank, "_name")]], name)
    expect_is(aajson$assessments, "data.frame")
  })

  test_that("fails well", {
    skip_on_cran()

    expect_error(fn(5), paste(rank, "must be of class character"))
    expect_error(fn(list()), paste(rank, "must be of class character"))

    expect_error(fn(key = 5), "key must be of class character")
    expect_error(fn(key = matrix()), "key must be of class character")

    expect_error(fn(parse = 5), "parse must be of class logical")
    expect_error(fn(parse = matrix()), "parse must be of class logical")

    expect_error(fn(page = "next"), "page must be of class integer")
    expect_error(fn(all = "yes"), "all must be of class logical")
    expect_error(fn(quiet = "no"), "quiet must be of class logical")

    # lengths
    expect_error(fn(page = 1:2), "page must be length 1")

  })
}

---
# Generated by vignettes/precompile.R: do not edit by hand.
# Please edit source in vignettes/source/_rredlist.Rmd
title: "Introduction to rredlist"
author: "William Gearty"
date: "2025-09-02"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to rredlist}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



`rredlist` is an R client for the [IUCN Red List API](https://api.iucnredlist.org/).
The [IUCN Red List](https://www.iucnredlist.org/) is the world’s most comprehensive information source on the global extinction risk status of animal, fungus and plant species. This package provides access via R to the various data contained within this database which span range details, population size, habitat and ecology, use and/or trade, threats, and conservation actions.

Documentation for the IUCN Red List API is available at https://api.iucnredlist.org/api-docs. **Note that use of the web API, and therefore this package, [requires authentication](#authentication).**

#### What `rredlist` is not:

1. [`redlistr`](https://cran.r-project.org/package=redlistr) is a different package for contributors to the IUCN Red List - not working with the IUCN Red List API.
2. `rredlist` does not include support for spatial data from the IUCN Red List. IUCN Red List Spatial data can be downloaded from the [IUCN website](https://www.iucnredlist.org/resources/spatial-data-download).

## Installation

CRAN


``` r
install.packages("rredlist")
```

Development version


``` r
# From GitHub
remotes::install_github("ropensci/rredlist")

# From r-universe
install.packages("rredlist", repos = "https://ropensci.r-universe.dev/")
```

## Authentication

IUCN requires you to get your own API key, an alphanumeric string that you need to send in every request. The rl_use_iucn() function in the package can walk you through [getting your own key](https://api.iucnredlist.org/users/sign_up) and storing it locally. Once you store this key, the functions in `rredlist` will always default to using this key.


``` r
rredlist::rl_use_iucn()
```

**Keep this key private.** You can pass the key in to each function via the `key` parameter, but it's better to store the key either as a environment variable (`IUCN_REDLIST_KEY`) or an R option (`iucn_redlist_key`) - we recommend using the former option.

## Overview of available features

- Query assessments by taxa (e.g., `rl_species()` and `rl_family()`)
- Query assessments by habitats (e.g., `rl_habitats()` and `rl_systems()`)
- Query assessments by geography (e.g., `rl_countries()` and `rl_realms()`)
- Query assessments by actions needed (e.g., `rl_actions()` and `rl_research()`)
- Query assessments by threat details (e.g., `rl_threats()` and `rl_pop_trends()`)
- Query assessments for specific sets of taxa (e.g., `rl_extinct()` and `rl_growth_forms()`)
- Get all details for a single assessment (e.g., `rl_species_latest()` and `rl_assessment()`)
- Red List information and statistics (e.g., `rl_sp_count()` and `rl_version()`)

## Example usage

### Loading the package

``` r
library("rredlist")
```

### Search for assessments for a particular species

``` r
rl_species("Gorilla", "gorilla")$assessments
#>    year_published latest possibly_extinct possibly_extinct_in_the_wild sis_taxon_id                                                url
#> 1            2016  FALSE            FALSE                        FALSE         9404  https://www.iucnredlist.org/species/9404/17963949
#> 2            2018   TRUE            FALSE                        FALSE         9404 https://www.iucnredlist.org/species/9404/136250858
#> 3            2016  FALSE            FALSE                        FALSE         9404 https://www.iucnredlist.org/species/9404/102330408
#> 4            2008  FALSE            FALSE                        FALSE         9404  https://www.iucnredlist.org/species/9404/12983787
#> 5            2007  FALSE            FALSE                        FALSE         9404  https://www.iucnredlist.org/species/9404/12983966
#> 6            2000  FALSE            FALSE                        FALSE         9404  https://www.iucnredlist.org/species/9404/12983737
#> 7            1996  FALSE            FALSE                        FALSE         9404  https://www.iucnredlist.org/species/9404/12983764
#> 8            1994  FALSE            FALSE                        FALSE         9404  https://www.iucnredlist.org/species/9404/12984167
#> 9            1990  FALSE            FALSE                        FALSE         9404  https://www.iucnredlist.org/species/9404/12984186
...
```

### Search for assessments that recommend particular conservation actions

#### Get a list of all conservation actions

``` r
rl_actions()
#> $conservation_actions
#>                                              en  code
#> 1                         Land/water protection     1
#> 2                          Site/area protection   1_1
#> 3                 Resource & habitat protection   1_2
#> 4                         Land/water management     2
#> 5                          Site/area management   2_1
#> 6          Invasive/problematic species control   2_2
#> 7         Habitat & natural process restoration   2_3
#> 8                            Species management     3
...
```

#### Return assessments with a particular conservation action

``` r
rl_actions("2_2", all = FALSE)$assessments
#>    year_published latest possibly_extinct possibly_extinct_in_the_wild sis_taxon_id
#> 1            2019   TRUE            FALSE                        FALSE    132523146
#> 2            2019   TRUE            FALSE                        FALSE        10767
#> 3            2013   TRUE            FALSE                        FALSE         1078
#> 4            2019   TRUE            FALSE                        FALSE    132521900
#> 5            2020  FALSE            FALSE                        FALSE         1086
#> 6            2019   TRUE            FALSE                        FALSE         1117
#> 7            2019  FALSE            FALSE                        FALSE        11797
#> 8            2021   TRUE            FALSE                        FALSE        12124
#> 9            2025   TRUE            FALSE                        FALSE    232775771
...
```

## Advanced usage

For a more in-depth walkthrough of how to integrate `rredlist` into research workflows, check out [the vignette](research_workflows.html).

## High level vs. low level interfaces

### High level interface


``` r
library("rredlist")
```

High level functions do the HTTP request and parse data to a list for ease
of downstream use. The high level functions have no underscore on the end
of the function name, e.g., `rl_species()`. Most of them return long lists containing lots of different information:


``` r
rl_species("Fratercula", "arctica")
#> $taxon
#> $taxon$sis_id
#> [1] 22694927
#> 
#> $taxon$scientific_name
#> [1] "Fratercula arctica"
#> 
#> $taxon$species_taxa
#> list()
#> 
...
```

By default, these high level functions will also parse the data to a data.frame, when possible:


``` r
rl_species("Fratercula", "arctica")$assessments
#>    year_published latest possibly_extinct possibly_extinct_in_the_wild sis_taxon_id
#> 1            2021   TRUE            FALSE                        FALSE     22694927
#> 2            2018   TRUE            FALSE                        FALSE     22694927
#> 3            2017  FALSE            FALSE                        FALSE     22694927
#> 4            2017  FALSE            FALSE                        FALSE     22694927
#> 5            2016  FALSE            FALSE                        FALSE     22694927
#> 6            2015  FALSE            FALSE                        FALSE     22694927
#> 7            2015  FALSE            FALSE                        FALSE     22694927
#> 8            2012  FALSE            FALSE                        FALSE     22694927
#> 9            2009  FALSE            FALSE                        FALSE     22694927
...
```

Likely a bit faster is to parse to a list only, and not take the extra data.frame parsing time:


``` r
rl_species("Fratercula", "arctica", parse = FALSE)$assessments
#> [[1]]
#> [[1]]$year_published
#> [1] "2021"
#> 
#> [[1]]$latest
#> [1] TRUE
#> 
#> [[1]]$possibly_extinct
#> [1] FALSE
#> 
...
```

For even more speed, you can use the low level package interface (although see the [benchmarking vignette](benchmarks.html) for specifics).

### Low level interface

The parsing to data.frame in the high level functions does take extra time. The low level functions
only do the HTTP request, and give back JSON without doing any more parsing. The low level functions DO have an underscore on the end of the function name, e.g., `rl_species_()`:


``` r
rl_species_("Fratercula", "arctica")
#> [1] "{\"taxon\":{\"sis_id\":22694927,\"scientific_name\":\"Fratercula arctica\",\"species_taxa\":[],\"subpopulation_taxa\":[],\"infrarank_taxa\":[],\"kingdom_name\":\"ANIMALIA\",\"phylum_name\":\"CHORDATA\",\"class_name\":\"AVES\",\"order_name\":\"CHARADRIIFORMES\",\"family_name\":\"ALCIDAE\",\"genus_name\":\"Fratercula\",\"species_name\":\"arctica\",\"subpopulation_name\":null,\"infra_name\":null,\"authority\":\"(Linnaeus, 1758)\",\"species\":true,\"subpopulation\":false,\"infrarank\":false,\"ssc_groups\":[{\"name\":\"IUCN SSC Bird Red List Authority (BirdLife International)\",\"url\":\"https://www.birdlife.org/\",\"description\":\"Red List Authority Coordinator: Ian Burfield (ian.burfield@birdlife.org)\"}],\"common_names\":[{\"main\":false,\"name\":\"Puffin\",\"language\":\"eng\"},{\"main\":true,\"name\":\"Atlantic Puffin\",\"language\":\"eng\"}],\"synonyms\":[]},\"assessments\":[{\"year_published\":\"2021\",\"latest\":true,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/166290968\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"EN\",\"assessment_id\":166290968,\"scopes\":[{\"description\":{\"en\":\"Europe\"},\"code\":\"2\"}]},{\"year_published\":\"2018\",\"latest\":true,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/132581443\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"VU\",\"assessment_id\":132581443,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"2017\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/117606911\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"VU\",\"assessment_id\":117606911,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"2017\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/110638141\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"VU\",\"assessment_id\":110638141,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"2016\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/93477427\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"VU\",\"assessment_id\":93477427,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"2015\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/82593109\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"VU\",\"assessment_id\":82593109,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"2015\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/60110592\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"EN\",\"assessment_id\":60110592,\"scopes\":[{\"description\":{\"en\":\"Europe\"},\"code\":\"2\"}]},{\"year_published\":\"2012\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/38908739\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"LC\",\"assessment_id\":38908739,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"2009\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/28178290\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"LC\",\"assessment_id\":28178290,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"2008\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/28179292\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"LC\",\"assessment_id\":28179292,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"2004\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/28180639\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"LC\",\"assessment_id\":28180639,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"2000\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/28181882\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"LR/lc\",\"assessment_id\":28181882,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"1994\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/28183212\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"LR/lc\",\"assessment_id\":28183212,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]},{\"year_published\":\"1988\",\"latest\":false,\"possibly_extinct\":false,\"possibly_extinct_in_the_wild\":false,\"sis_taxon_id\":22694927,\"url\":\"https://www.iucnredlist.org/species/22694927/28184380\",\"taxon_scientific_name\":\"Fratercula arctica\",\"red_list_category_code\":\"LR/lc\",\"assessment_id\":28184380,\"scopes\":[{\"description\":{\"en\":\"Global\"},\"code\":\"1\"}]}],\"params\":{\"genus_name\":\"Fratercula\",\"species_name\":\"arctica\"}}"
```

To consume this JSON, you can use [`jsonlite`](https://jeroen.r-universe.dev/jsonlite):


``` r
library("jsonlite")
fromJSON(rl_species_("Fratercula", "arctica"))
#> $taxon
#> $taxon$sis_id
#> [1] 22694927
#> 
#> $taxon$scientific_name
#> [1] "Fratercula arctica"
#> 
#> $taxon$species_taxa
#> list()
#> 
...
```

## Usage best practice

### Citing the IUCN Red List API

Use the function `rl_citation()`:


``` r
rl_citation()
#> To cite the IUCN API in publications, use the following citation:
#> 
#>   IUCN (2025). "IUCN Red List of Threatened Species." <https://www.iucnredlist.org>. Accessed on 02 September 2025.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Misc{IUCN2025,
#>     author = {{IUCN}},
#>     title = {IUCN Red List of Threatened Species},
#>     year = {2025},
#>     edition = {Version 2025-1},
#>     howpublished = {\url{https://www.iucnredlist.org}},
#>     note = {Accessed on 02 September 2025},
#>   }
```

We'd also really appreciate it if you could cite your use of this package:


``` r
citation("rredlist")
#> To cite package 'rredlist' in publications use:
#> 
#>   Gearty W, Chamberlain S (2025). _rredlist: 'IUCN' Red List Client_. doi:10.32614/CRAN.package.rredlist
#>   <https://doi.org/10.32614/CRAN.package.rredlist>, R package version 1.1.0, <https://docs.ropensci.org/rredlist/>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Manual{rredlist,
#>     title = {rredlist: 'IUCN' Red List Client},
#>     author = {William Gearty and Scott Chamberlain},
#>     year = {2025},
#>     note = {R package version 1.1.0},
#>     doi = {10.32614/CRAN.package.rredlist},
#>     url = {https://docs.ropensci.org/rredlist/},
#>   }
```

### Rate Limiting

From the IUCN folks: "Too many frequent calls, or too many calls per day
might get your access blocked temporarily. If you're a heavy API user, the
Red List Unit asked that you contact them, as there might be better options.
They suggest a 2-second delay between your calls if you plan to make a
lot of calls."

### API Versioning

`rredlist` versions ≤ 0.7.1 tracked the now-defunct version 3 of the IUCN Red List API. `rredlist` versions ≥ 1.0.0 track version 4 of the IUCN Red List API. If you need to use version 3 of the API--and it is still functioning--you can install an old version of `rredlist` using the `remotes` package:


``` r
# From CRAN archive
remotes::install_version("rredlist", version = "0.7.1")

# From r-universe
remotes::install_version("rredlist", version = "0.7.1",
                         repos = "https://ropensci.r-universe.dev/")
```

Note, however, that you can no longer generate a new API key for version 3 of the API. See the discussion [here](https://github.com/ropensci/rredlist/issues/52) for possible solutions if you need to use version 3 of the API.

---
# Generated by vignettes/precompile.R: do not edit by hand.
# Please edit source in vignettes/source/_benchmarks.Rmd
title: "rredlist benchmarks"
author: "William Gearty"
date: "2025-09-02"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{rredlist benchmarks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



<div style="text-align: justify">

## Introduction

`rredlist` provides two APIs, a higher-level one that takes slightly more time but returns the data in a more user-friendly format (a list), and a lower-level one (i.e., functions that end with "_") that takes less time but does no processing of the data (returning the raw JSON string). Both APIs return the exact same information, but it is up to the user whether the format processing is worth the extra time, especially when performing bulk operations. To help inform this decision by the user, here is some benchmarking related to the two APIs. First, we'll break down the total difference in computation time between the two APIs, then we'll dig into what components are causing this difference. We'll use `microbenchmark::microbenchmark()` which has very little computational overhead. Note that the time units vary from comparison to comparison, and the speed of these functions may be highly hardware- and network-dependent.


``` r
library(rredlist)
library(microbenchmark)
```

## Head-to-head benchmarks

We'll start by benchmarking the two APIs head-to-head. We'll test a couple of use cases, in rough order of increasing complexity.

#### 1\. Get species count


``` r
microbenchmark(
  rl_sp_count(),
  rl_sp_count_(),
  times = 10
)
#> Unit: milliseconds
#>            expr      min       lq     mean   median       uq      max neval cld
#>   rl_sp_count() 116.7317 119.2230 120.2650 119.9624 120.2204 127.7930    10   a
#>  rl_sp_count_() 116.9935 117.4514 142.3724 118.6451 120.1328 313.6149    10   a
```

#### 2\. Lookup individual assessment


``` r
microbenchmark(
  rl_assessment(136250858),
  rl_assessment_(136250858),
  times = 10
)
#> Unit: milliseconds
#>                       expr      min       lq     mean   median      uq      max neval cld
#>   rl_assessment(136250858) 240.1841 243.4012 250.3891 248.4046 255.671 265.0165    10   a
#>  rl_assessment_(136250858) 239.4693 242.0733 260.7446 247.0122 249.948 388.4490    10   a
```

#### 3\. Taxonomic lookup with defaults


``` r
microbenchmark(
  rl_family(),
  rl_family_(),
  times = 10
)
#> Unit: milliseconds
#>          expr      min       lq     mean   median       uq      max neval cld
#>   rl_family() 126.6563 126.9070 131.7954 127.7793 128.5488 158.2391    10   a
#>  rl_family_() 126.2004 129.5163 132.9154 131.4624 136.4402 141.6659    10   a
```

#### 4\. Taxonomic lookup with query (one page of results)


``` r
microbenchmark(
  rl_family("Rheidae"),
  rl_family_("Rheidae"),
  times = 10
)
#> Unit: milliseconds
#>                   expr      min       lq     mean   median       uq      max neval cld
#>   rl_family("Rheidae") 614.5653 628.8536 672.6828 669.7144 712.9704 739.9620    10   a
#>  rl_family_("Rheidae") 606.4974 642.0678 693.4926 676.5411 756.3828 822.1573    10   a
```

#### 5\. Taxonomic lookup with query (~10 pages of results)


``` r
microbenchmark(
  rl_family("Corvidae", quiet = TRUE),
  rl_family_("Corvidae", quiet = TRUE),
  times = 10
)
#> Unit: seconds
#>                                  expr      min       lq     mean   median       uq      max neval cld
#>   rl_family("Corvidae", quiet = TRUE) 10.89607 11.02283 11.18194 11.17454 11.32452 11.48761    10   a
#>  rl_family_("Corvidae", quiet = TRUE) 10.83396 10.92033 11.09111 11.04066 11.16850 11.63398    10   a
```

#### 6\. Taxonomic lookup with query (~40 pages of results)


``` r
microbenchmark(
  rl_family("Tyrannidae", quiet = TRUE),
  rl_family_("Tyrannidae", quiet = TRUE),
  times = 10
)
#> Unit: seconds
#>                                    expr      min       lq     mean   median       uq      max neval cld
#>   rl_family("Tyrannidae", quiet = TRUE) 38.17690 38.26825 38.75878 38.58874 38.93320 39.98190    10   a
#>  rl_family_("Tyrannidae", quiet = TRUE) 37.74677 37.93254 38.28780 38.00963 38.72292 39.07203    10   a
```

#### 7\. Taxonomic lookup with query (~900 pages of results)


``` r
microbenchmark(
  rl_class("Aves", quiet = TRUE),
  rl_class_("Aves", quiet = TRUE),
  times = 10
)
#> Unit: seconds
#>                             expr      min       lq     mean   median       uq      max neval cld
#>   rl_class("Aves", quiet = TRUE) 1424.662 1451.755 1467.593 1468.110 1483.732 1513.186    10   a
#>  rl_class_("Aves", quiet = TRUE) 1428.810 1435.013 1448.085 1444.788 1465.589 1472.253    10   a
```

### And the winner is...

As you can see above, the two APIs take roughly the same amount of time for most use cases. I previously said that the low-level API is designed to be faster. While most of these comparisons agree with that statement, the time reduction is usually a few milliseconds per function call. When we get into more complex queries, like returning multiple pages of API results, we start to see larger time reductions, especially as the number of pages of results increases (10+ seconds for hundreds of pages).

## Query breakdown

Based on the above, it doesn't seem to matter much, time-wise, whether we parse the data or not. So then what takes up all of the query time? Let's break down the process of querying the API and downloading a single page of assessments using some of the internal functions of `rredlist`:




``` r
microbenchmark(
  res <- rredlist:::rr_GET_raw("taxa/family/Rheidae"), # get the raw data for the first page
  x <- res$parse("UTF-8"), # parse the raw response data to JSON
  rredlist:::rl_parse(x, parse = TRUE), # parse the JSON to a list of dataframes
  rredlist:::rl_parse(x, parse = FALSE), # parse the JSON to a list of lists
  times = 10
)
#> Unit: microseconds
#>                                                 expr      min       lq      mean    median       uq      max neval cld
#>  res <- rredlist:::rr_GET_raw("taxa/family/Rheidae") 588296.5 595734.3 607564.29 603497.10 620369.4 634292.7    10  a 
#>                              x <- res$parse("UTF-8")    807.9    829.0   1971.02   1028.45   2543.7   7116.8    10   b
#>                 rredlist:::rl_parse(x, parse = TRUE)   1021.6   1159.5   1440.46   1212.80   1302.7   3237.3    10   b
#>                rredlist:::rl_parse(x, parse = FALSE)     76.4     97.2    167.72    114.65    292.4    317.2    10   b
```

The above benchmarking shows us that the vast majority of time is spent downloading data from the IUCN API. For a single page of results, even the highest level of parsing takes only 0.15% of the time it takes to download the data. Further, while parsing to a list of dataframes (`parse = TRUE`) takes about 10 times as long as just parsing to a list of lists (`parse = FALSE`), both methods remain very quick compared to the process of downloading the data.

Now let's break down a multi-page query:




``` r
microbenchmark(
  lst <- rredlist:::page_assessments("taxa/family/Tyrannidae",
                                     key = rredlist:::check_key(NULL),
                                     quiet = TRUE), # get the data for all of the pages
  rredlist:::combine_assessments(lst, parse = TRUE), # parse the JSON to a list of dataframes
  rredlist:::combine_assessments(lst, parse = FALSE), # parse the JSON to a list of lists
  times = 10
)
#> Unit: milliseconds
#>                                                                                                               expr        min         lq
#>  lst <- rredlist:::page_assessments("taxa/family/Tyrannidae",      key = rredlist:::check_key(NULL), quiet = TRUE) 36737.2111 36941.2153
#>                                                                  rredlist:::combine_assessments(lst, parse = TRUE)   249.1083   266.0197
#>                                                                 rredlist:::combine_assessments(lst, parse = FALSE)    13.7986    15.4881
#>         mean      median         uq        max neval cld
#>  37558.35359 37456.89045 38066.7995 39157.4033    10  a 
#>    272.17091   271.24645   279.7633   291.9884    10   b
#>     16.63622    15.62825    16.6812    23.0368    10   b
```

Again, even with about 40 pages of data to parse, the download takes the vast majority of the time. The highest-level parsing has increased to about 1% of the time it takes to download the data, but this remains less than a second compared to the ~35 second download.

## Conclusion

Ultimately, both APIs take about the same amount of time because the majority of time is spent downloading the data from the IUCN database and reading it into R. For larger downloads, the parsing done by the high-level API may take an appreciable amount of time (tenths of seconds to seconds). It's possible that users who are calling these functions many (e.g., thousands) of times would appreciate this time reduction. However, for most users it probably won't matter. Furthermore, keep in mind that if you use the low-level API you will likely need to do your own processing after the fact in order to do any sort of downstream analyses. Ultimately, the choice is up to you.

</div>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_key.R
\name{rl_use_iucn}
\alias{rl_use_iucn}
\alias{rl_key}
\title{Helper to get and save IUCN API key}
\usage{
rl_use_iucn()
}
\value{
Invisibly returns the sign-up URL for the IUCN Red List API.
}
\description{
Provides instruction on how to get and store a key for the IUCN
API. Users of \code{rredlist} must do this before using any of the package's
other functionality. Note that registering for a key requires an email
address and information about your organization.
}
\details{
Note that after filling the online form, you should receive an API
key shortly but not immediately.
}
\examples{
\dontrun{
# Sign up for an API key
rl_use_iucn()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_assessment.R
\name{rl_assessment_list}
\alias{rl_assessment_list}
\title{Retrieve a list of assessments}
\usage{
rl_assessment_list(ids, key = NULL, wait_time = 0.5, quiet = FALSE, ...)
}
\arguments{
\item{ids}{(integer) A vector of unique identifiers of the assessments.}

\item{key}{(character) An IUCN API token. See \code{\link[=rl_use_iucn]{rl_use_iucn()}}.}

\item{wait_time}{(numeric) The number of seconds to wait between queries. The
default is 0.5 seconds, which is recommended by IUCN to avoid timeouts.}

\item{quiet}{(logical) If \code{TRUE}, suppresses the progress bar.}

\item{...}{Includes the following arguments related to the wait time between
request retries if a "Too Many Requests" error is received from the API
(see \link[crul:HttpClient]{HttpClient()$retry()} for more details):
\itemize{
\item \code{pause_base}, \code{pause_cap}, and \code{pause_min}: basis, maximum, and minimum
for calculating wait time for retry
\item \code{times}: the maximum number of times to retry
\item \code{onwait}: a callback function if the request will be retried and a
wait time is being applied
}
Also supports any \link[curl:curl_options]{curl options} passed to the GET
request via \link[crul:HttpClient]{HttpClient}.}
}
\value{
A list with each element representing the response of
\code{\link[=rl_assessment]{rl_assessment()}}.
}
\description{
Get the full details for a list of IUCN Red List assessments. This is a
wrapper for \code{\link[=rl_assessment]{rl_assessment()}} that allows you to retrieve multiple
assessments at once.
}
\examples{
\dontrun{
# Get assessment details for multiple assessments
ex1 <- rl_assessment_list(ids = c(166290968, 136250858))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_taxa.R
\name{rl_species_latest}
\alias{rl_species_latest}
\title{Species latest assessment}
\usage{
rl_species_latest(
  genus,
  species,
  infra = NULL,
  subpopulation = NULL,
  scope = "1",
  key = NULL,
  parse = TRUE,
  ...
)
}
\arguments{
\item{genus}{(character) The genus name of the species to look up.}

\item{species}{(character) The species epithet of the species to look up.}

\item{infra}{(character) An optional name of the subspecies or variety to
look up.}

\item{subpopulation}{(character) An optional name of the geographically
separate subpopulation to look up.}

\item{scope}{(character) An optional scope (see \code{\link[=rl_scopes]{rl_scopes()}}) to filter the
results by. The default, "1", returns the latest global assessment. If no
assessments exist for the specified \code{scope}, the latest assessment across
all scopes is returned. If \code{NULL}, the latest assessment across all scopes
is returned.}

\item{key}{(character) An IUCN API token. See \code{\link[=rl_use_iucn]{rl_use_iucn()}}.}

\item{parse}{(logical) Whether to parse the output to list (\code{FALSE}) or,
where possible, data.frame (\code{TRUE}). Default: \code{TRUE}.}

\item{...}{Includes the following arguments related to the wait time between
request retries if a "Too Many Requests" error is received from the API
(see \link[crul:HttpClient]{HttpClient()$retry()} for more details):
\itemize{
\item \code{pause_base}, \code{pause_cap}, and \code{pause_min}: basis, maximum, and minimum
for calculating wait time for retry
\item \code{times}: the maximum number of times to retry
\item \code{onwait}: a callback function if the request will be retried and a
wait time is being applied
}
Also supports any \link[curl:curl_options]{curl options} passed to the GET
request via \link[crul:HttpClient]{HttpClient}.}
}
\value{
A list unless using a function with a trailing underscore, in which
case json as character string is returned.
}
\description{
Get the latest assessment for a particular species (i.e., Latin binomial) or
subspecies/variety/subpopulation (i.e., Latin trinomial). Wraps
\code{\link[=rl_species]{rl_species()}} and \code{\link[=rl_assessment]{rl_assessment()}}.
}
\details{
Geographically separate subpopulations of a species are defined as
those populations that are so isolated from others of the same species that
it is considered extremely unlikely that there is any genetic interchange.
In general, listings of such subpopulations are restricted to those that
have been isolated for a long period of time.

Assessments of subspecies, varieties, and geographically separate
subpopulations must adhere to the same standards as for species
assessments. However, these assessments are only included provided there is
a global assessment of the species as a whole.

Infraspecific ranks such as formas, subvarieties, cultivars, etc are not
included in the Red List.
}
\examples{
\dontrun{
# Get latest assessment for species
ex1 <- rl_species_latest(genus = "Fratercula", species = "arctica")
ex1$stresses

# Get latest assessment for subspecies
ex2 <- rl_species_latest(genus = "Gorilla", species = "gorilla",
                         infra = "gorilla")
ex2$stresses
}
}
\references{
API docs at \url{https://api.iucnredlist.org/}.
}
\seealso{
Assessments by taxa:
\code{\link{rl_class}()},
\code{\link{rl_family}()},
\code{\link{rl_kingdom}()},
\code{\link{rl_order}()},
\code{\link{rl_phylum}()},
\code{\link{rl_sis}()},
\code{\link{rl_sis_latest}()},
\code{\link{rl_species}()}
}
\concept{taxa}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_categories.R
\name{rl_categories}
\alias{rl_categories}
\alias{rl_categories_}
\title{Red List category assessment summary}
\usage{
rl_categories(
  code = NULL,
  key = NULL,
  parse = TRUE,
  all = TRUE,
  page = 1,
  quiet = FALSE,
  ...
)

rl_categories_(
  code = NULL,
  key = NULL,
  all = TRUE,
  page = 1,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{code}{(character) The code of the Red List category to look up. If not
supplied, a list of all categories will be returned.}

\item{key}{(character) An IUCN API token. See \code{\link[=rl_use_iucn]{rl_use_iucn()}}.}

\item{parse}{(logical) Whether to parse the output to list (\code{FALSE}) or,
where possible, data.frame (\code{TRUE}). Default: \code{TRUE}.}

\item{all}{(logical) Whether to retrieve all results at once or not. If
\code{TRUE} we do the paging internally for you and bind all of the results
together. If \code{FALSE}, only a single page of results will be retrieved.}

\item{page}{(integer/numeric) Page to get if \code{all} is \code{FALSE}. Default: 1.
Each page returns up to 100 records. Paging is required because it's too
much burden on a server to just "get all the data" in one request.}

\item{quiet}{(logical) Whether to suppress progress for multi-page downloads
or not. Default: \code{FALSE} (that is, give progress). Ignored if \code{all = FALSE}.}

\item{...}{Includes the following filters that are supported by the API
endpoint (refer to the \href{https://api.iucnredlist.org/api-docs/index.html}{API docs} for more
information):
\itemize{
\item \code{year_published}: (integer) Set this to return only
assessments from a given year.
\item \code{latest}: (logical) Set this to \code{TRUE} to return only the
latest assessment for each taxon.
\item \code{scope_code}: (integer) Set this to return only assessments
from a particular \link[=rl_scopes]{scope} (e.g., \code{1} for Global, \code{2} for
Europe). This is similar to the \code{region} argument of the old Red List
API and old versions of rredlist.
}
Also includes the following arguments related to the wait time between
request retries if a "Too Many Requests" error is received from the API
(see \link[crul:HttpClient]{HttpClient()$retry()} for more details):
\itemize{
\item \code{pause_base}, \code{pause_cap}, and \code{pause_min}: basis, maximum, and minimum
for calculating wait time for retry
\item \code{times}: the maximum number of times to retry
\item \code{onwait}: a callback function if the request will be retried and a
wait time is being applied
}
Also supports any \link[curl:curl_options]{curl options} passed to the GET
request via \link[crul:HttpClient]{HttpClient}.}
}
\value{
A list unless using a function with a trailing underscore, in which
case json as character string is returned.
}
\description{
Return a list of the latest assessments for a given category (e.g., Not
Evaluated, Data Deficient, Least Concern, Near Threatened, Vulnerable,
Endangered, Critically Endangered, Extinct in the Wild and Extinct). This
endpoint returns categories for the current IUCN Red List Categories and
Criteria (v3.1) as well as older versions (i.e., v2.3). Note that a code may
not be unique across Categories and Criteria versions. Therefore, codes like
"EX" will return assessments for EX v3.1 and EX v2.3.
}
\examples{
\dontrun{
# Get all Red List categories
rl_categories()
# Get assessments for the Vulnerable category
rl_categories("VU")
}
}
\references{
API docs at \url{https://api.iucnredlist.org/}.
}
\seealso{
Assessments by threat details:
\code{\link{rl_pop_trends}()},
\code{\link{rl_stresses}()},
\code{\link{rl_threats}()},
\code{\link{rl_use_and_trade}()}
}
\concept{threats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_assessment.R
\name{rl_assessment}
\alias{rl_assessment}
\alias{rl_assessment_}
\title{Retrieve an assessment}
\usage{
rl_assessment(id, key = NULL, parse = TRUE, ...)

rl_assessment_(id, key = NULL, ...)
}
\arguments{
\item{id}{(integer) The unique identifier of the assessment.}

\item{key}{(character) An IUCN API token. See \code{\link[=rl_use_iucn]{rl_use_iucn()}}.}

\item{parse}{(logical) Whether to parse the output to list (\code{FALSE}) or,
where possible, data.frame (\code{TRUE}). Default: \code{TRUE}.}

\item{...}{Includes the following arguments related to the wait time between
request retries if a "Too Many Requests" error is received from the API
(see \link[crul:HttpClient]{HttpClient()$retry()} for more details):
\itemize{
\item \code{pause_base}, \code{pause_cap}, and \code{pause_min}: basis, maximum, and minimum
for calculating wait time for retry
\item \code{times}: the maximum number of times to retry
\item \code{onwait}: a callback function if the request will be retried and a
wait time is being applied
}
Also supports any \link[curl:curl_options]{curl options} passed to the GET
request via \link[crul:HttpClient]{HttpClient}.}
}
\value{
A list unless using a function with a trailing underscore, in which
case json as character string is returned.
}
\description{
Get the full details for a single IUCN Red List assessment.
}
\examples{
\dontrun{
# Get assessment details for Fratercula arctica
ex1 <- rl_assessment(id = 166290968)
ex1$red_list_category$code
ex1$systems
}
}
\references{
API docs at \url{https://api.iucnredlist.org/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defunct.R
\name{rl_sp_country}
\alias{rl_sp_country}
\alias{rl_sp_country_}
\title{Get species by country}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

This function is fully deprecated in favor of \code{\link[=rl_countries]{rl_countries()}} as of
\strong{rredlist} version 1.0.0.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_taxa.R
\name{rl_sis_latest}
\alias{rl_sis_latest}
\title{SIS ID latest assessment}
\usage{
rl_sis_latest(id, scope = "1", key = NULL, parse = TRUE, ...)
}
\arguments{
\item{id}{(integer) The SIS ID of the taxonomic entity to look up.}

\item{scope}{(character) An optional scope (see \code{\link[=rl_scopes]{rl_scopes()}}) to filter the
results by. The default, "1", returns the latest global assessment. If no
assessments exist for the specified \code{scope}, the latest assessment across
all scopes is returned. If \code{NULL}, the latest assessment across all scopes
is returned.}

\item{key}{(character) An IUCN API token. See \code{\link[=rl_use_iucn]{rl_use_iucn()}}.}

\item{parse}{(logical) Whether to parse the output to list (\code{FALSE}) or,
where possible, data.frame (\code{TRUE}). Default: \code{TRUE}.}

\item{...}{Includes the following arguments related to the wait time between
request retries if a "Too Many Requests" error is received from the API
(see \link[crul:HttpClient]{HttpClient()$retry()} for more details):
\itemize{
\item \code{pause_base}, \code{pause_cap}, and \code{pause_min}: basis, maximum, and minimum
for calculating wait time for retry
\item \code{times}: the maximum number of times to retry
\item \code{onwait}: a callback function if the request will be retried and a
wait time is being applied
}
Also supports any \link[curl:curl_options]{curl options} passed to the GET
request via \link[crul:HttpClient]{HttpClient}.}
}
\value{
A list unless using a function with a trailing underscore, in which
case json as character string is returned.
}
\description{
Get the latest assessment for a particular taxonomic entity based on its ID
number from the \href{https://www.iucnredlist.org/assessment/sis}{IUCN Species Information Service (SIS)}. Wraps \code{\link[=rl_sis]{rl_sis()}} and
\code{\link[=rl_assessment]{rl_assessment()}}.
}
\examples{
\dontrun{
# Get latest assessment for species
ex1 <- rl_sis_latest(id = 9404)
ex1$stresses
}
}
\references{
API docs at \url{https://api.iucnredlist.org/}.
}
\seealso{
Assessments by taxa:
\code{\link{rl_class}()},
\code{\link{rl_family}()},
\code{\link{rl_kingdom}()},
\code{\link{rl_order}()},
\code{\link{rl_phylum}()},
\code{\link{rl_sis}()},
\code{\link{rl_species}()},
\code{\link{rl_species_latest}()}
}
\concept{taxa}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_version.R
\name{rl_api_version}
\alias{rl_api_version}
\title{Get the Red List API version}
\usage{
rl_api_version(key = NULL, ...)
}
\arguments{
\item{key}{(character) An IUCN API token. See \code{\link[=rl_use_iucn]{rl_use_iucn()}}.}

\item{...}{Curl options passed to \link[crul:HttpClient]{HttpClient}.}
}
\value{
API version as character string.
}
\description{
Returns the current version number of the IUCN Red List of Threatened Species
API.
}
\examples{
\dontrun{
rl_api_version()
}
}
\seealso{
Red List information and statistics:
\code{\link{rl_citation}()},
\code{\link{rl_sp_count}()},
\code{\link{rl_version}()}
}
\concept{stats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_realms.R
\name{rl_realms}
\alias{rl_realms}
\alias{rl_realms_}
\title{Biogeographical realm assessment summary}
\usage{
rl_realms(
  code = NULL,
  key = NULL,
  parse = TRUE,
  all = TRUE,
  page = 1,
  quiet = FALSE,
  ...
)

rl_realms_(code = NULL, key = NULL, all = TRUE, page = 1, quiet = FALSE, ...)
}
\arguments{
\item{code}{(character) The code of the biogeographical realm to look up. If
not supplied, a list of all biogeographical realms will be returned.}

\item{key}{(character) An IUCN API token. See \code{\link[=rl_use_iucn]{rl_use_iucn()}}.}

\item{parse}{(logical) Whether to parse the output to list (\code{FALSE}) or,
where possible, data.frame (\code{TRUE}). Default: \code{TRUE}.}

\item{all}{(logical) Whether to retrieve all results at once or not. If
\code{TRUE} we do the paging internally for you and bind all of the results
together. If \code{FALSE}, only a single page of results will be retrieved.}

\item{page}{(integer/numeric) Page to get if \code{all} is \code{FALSE}. Default: 1.
Each page returns up to 100 records. Paging is required because it's too
much burden on a server to just "get all the data" in one request.}

\item{quiet}{(logical) Whether to suppress progress for multi-page downloads
or not. Default: \code{FALSE} (that is, give progress). Ignored if \code{all = FALSE}.}

\item{...}{Includes the following filters that are supported by the API
endpoint (refer to the \href{https://api.iucnredlist.org/api-docs/index.html}{API docs} for more
information):
\itemize{
\item \code{year_published}: (integer) Set this to return only
assessments from a given year.
\item \code{latest}: (logical) Set this to \code{TRUE} to return only the
latest assessment for each taxon.
\item \code{scope_code}: (integer) Set this to return only assessments
from a particular \link[=rl_scopes]{scope} (e.g., \code{1} for Global, \code{2} for
Europe). This is similar to the \code{region} argument of the old Red List
API and old versions of rredlist.
}
Also includes the following arguments related to the wait time between
request retries if a "Too Many Requests" error is received from the API
(see \link[crul:HttpClient]{HttpClient()$retry()} for more details):
\itemize{
\item \code{pause_base}, \code{pause_cap}, and \code{pause_min}: basis, maximum, and minimum
for calculating wait time for retry
\item \code{times}: the maximum number of times to retry
\item \code{onwait}: a callback function if the request will be retried and a
wait time is being applied
}
Also supports any \link[curl:curl_options]{curl options} passed to the GET
request via \link[crul:HttpClient]{HttpClient}.}
}
\value{
A list unless using a function with a trailing underscore, in which
case json as character string is returned.
}
\description{
Get an assessment summary for a particular biogeographical realm (e.g.,
Neotropical or Palearctic).
}
\examples{
\dontrun{
# Get a list of all biogeographical realms
rl_realms()
# Get assessment summary for the Afrotropical biogeographical realm
rl_realms(code = "0")
}
}
\references{
API docs at \url{https://api.iucnredlist.org/}.
}
\seealso{
Assessments by geographic regions:
\code{\link{rl_countries}()},
\code{\link{rl_faos}()},
\code{\link{rl_scopes}()}
}
\concept{geo}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_categories.R
\name{scale_color_iucn}
\alias{scale_color_iucn}
\alias{scale_colour_iucn}
\alias{scale_fill_iucn}
\alias{scale_discrete_iucn}
\title{IUCN Red List color scales}
\usage{
scale_color_iucn(...)

scale_fill_iucn(...)

scale_discrete_iucn(aesthetics, ...)
}
\arguments{
\item{...}{Arguments passed on to \link[ggplot2:discrete_scale]{ggplot2::discrete_scale} (except for
\code{na.value} which is already set according to the IUCN standard of
\verb{#C1B5A5})}

\item{aesthetics}{Character string or vector of character strings listing the
name(s) of the aesthetic(s) that this scale works with. This can be useful,
for example, to apply color settings to the \code{color} and \code{fill} aesthetics
at the same time, via \code{aesthetics = c("color", "fill")}.}
}
\description{
ggplot2 color scales using the colors for the IUCN Red List threat categories, as
documented in \href{https://nc.iucnredlist.org/redlist/resources/files/1646067752-FINAL_IUCN_Red_List_colour_chart.pdf}{this chart}.
}
\examples{
\dontshow{if (require(ggplot2)) withAutoprint(\{ # examplesIf}
library(ggplot2)
categories <- c("NE", "DD", "LC", "NT", "VU", "EN", "CR", "RE", "EW", "EX")
df <- data.frame(
  x = runif(1000, 0, 10), y = runif(1000, 0, 10),
  color = sample(categories, 1000, TRUE), shape = 21
)
ggplot(df) +
  geom_point(aes(x = x, y = y, fill = color), shape = 21) +
  scale_fill_iucn(name = "IUCN Category") +
  theme_classic()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rredlist-package.R
\docType{package}
\name{rredlist-package}
\alias{rredlist-package}
\alias{rredlist}
\title{rredlist: 'IUCN' Red List Client}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

'IUCN' Red List (\url{https://api.iucnredlist.org/}) client. The 'IUCN' Red List is a global list of threatened and endangered species. Functions cover all of the Red List 'API' routes. An 'API' key is required.
}
\section{Authentication}{

IUCN requires you to get your own API key, an alphanumeric string that you
need to send in every request. See \code{\link[=rl_use_iucn]{rl_use_iucn()}} for help getting and
storing it. Get it at \url{https://api.iucnredlist.org/users/sign_up}. Keep this
key private. You can pass the key in to each function via the \code{key}
parameter, but it's better to store the key either as an environment variable
(\code{IUCN_REDLIST_KEY}) or an R option (\code{iucn_redlist_key}) - we recommend using
the former option.
}

\section{High vs. Low level package APIs}{

\strong{High level API}
High level functions do the HTTP request and parse data to a data.frame for
ease of downstream use. The high level functions have no underscore on
the end of the function name, e.g., \code{\link[=rl_species]{rl_species()}}.

\strong{Low level API}
The parsing to data.frame in the high level API does take extra time.
The low level API only does the HTTP request, and gives back JSON without
doing any more parsing. The low level functions DO have an underscore on
the end of the function name, e.g., \code{\link[=rl_species_]{rl_species_()}}.
}

\section{No Spatial}{

This package does not include support for spatial data from the IUCN Red
List. IUCN Red List Spatial data can be downloaded at
\url{https://www.iucnredlist.org/resources/spatial-data-download}.
}

\section{Citing the IUCN Red List API}{

Get the proper citation for the version of the Red List you are using
by programatically running \code{\link[=rl_citation]{rl_citation()}}. Also see
\url{https://api.iucnredlist.org/}.
}

\section{Red List API Terms of Use}{

See \url{https://www.iucnredlist.org/terms/terms-of-use}.
}

\section{Rate limiting}{

From the IUCN folks: Too many frequent calls, or too many calls per day
might get your access blocked temporarily. If you're a heavy API user, the
Red List Unit asked that you contact them, as there might be better options.
They suggest a 2-second delay between your calls if you plan to make a
lot of calls.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://docs.ropensci.org/rredlist/}
  \item \url{https://github.com/ropensci/rredlist}
  \item Report bugs at \url{https://github.com/ropensci/rredlist/issues}
}

}
\author{
\strong{Maintainer}: William Gearty \email{willgearty@gmail.com} (\href{https://orcid.org/0000-0003-0076-3262}{ORCID})

Authors:
\itemize{
  \item Scott Chamberlain \email{myrmecocystus@gmail.com} (\href{https://orcid.org/0000-0003-1444-9135}{ORCID})
}

Other contributors:
\itemize{
  \item rOpenSci (\href{https://ror.org/019jywm96}{ROR}) [funder]
  \item Maëlle Salmon \email{maelle.salmon@yahoo.se} (\href{https://orcid.org/0000-0002-2815-0399}{ORCID}) [contributor]
  \item Kevin Cazelles \email{kevin.cazelles@insileco.io} (\href{https://orcid.org/0000-0001-6619-9874}{ORCID}) [reviewer]
  \item Stephanie Hazlitt \email{stephhazlitt@gmail.com} (\href{https://orcid.org/0000-0002-3161-2304}{ORCID}) [reviewer]
}

}
\keyword{internal}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_citation.R
\name{rl_citation}
\alias{rl_citation}
\title{Get the citation for the Red List API}
\usage{
rl_citation(key = NULL, ...)
}
\arguments{
\item{key}{(character) An IUCN API token. See \code{\link[=rl_use_iucn]{rl_use_iucn()}}.}

\item{...}{Curl options passed to \link[crul:HttpClient]{HttpClient}.}
}
\value{
Red List citation as a \link[utils:bibentry]{bibentry} object.
}
\description{
Full acknowledgement and citation needs to be given for using the API. Use
this function to get the full citation for the current version of the Red
List API. More details are available here: \url{https://api.iucnredlist.org/}.
}
\examples{
\dontrun{
rl_citation()
}
}
\seealso{
Red List information and statistics:
\code{\link{rl_api_version}()},
\code{\link{rl_sp_count}()},
\code{\link{rl_version}()}
}
\concept{stats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_sp_count.R
\name{rl_sp_count}
\alias{rl_sp_count}
\alias{rl_sp_count_}
\title{Get count of species in the Red List}
\usage{
rl_sp_count(key = NULL, ...)

rl_sp_count_(key = NULL, ...)
}
\arguments{
\item{key}{(character) An IUCN API token. See \code{\link[=rl_use_iucn]{rl_use_iucn()}}.}

\item{...}{Curl options passed to \link[crul:HttpClient]{HttpClient}}
}
\value{
An integer representing the number of unique species represented
within the IUCN database.
}
\description{
Returns a count of the number of unique species which have assessments.
}
\examples{
\dontrun{
# Get count of species with assessments
rl_sp_count()
}
}
\references{
API docs at \url{https://api.iucnredlist.org/}.
}
\seealso{
Red List information and statistics:
\code{\link{rl_api_version}()},
\code{\link{rl_citation}()},
\code{\link{rl_version}()}
}
\concept{stats}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rl_assessment.R
\name{rl_assessment_extract}
\alias{rl_assessment_extract}
\title{Extract an assessment element}
\usage{
rl_assessment_extract(lst, el_name, format = c("list", "df"), flatten = FALSE)
}
\arguments{
\item{lst}{(list) A list of assessments, as returned by
\code{\link[=rl_assessment_list]{rl_assessment_list()}}. If \code{lst} contains any \code{NULL} elements, they will be
removed.}

\item{el_name}{(character) The name of the element to extract from each
assessment. Supports multilevel extraction using "__" as the separator. For
example, to extract the synonyms table, you could use "taxon__synonyms".
\code{el_name} may also be a character vector indicating multiple elements to
extract. In this case, the output will contain all requested elements,
either as a list (if \code{format = "list"}) or as a data.frame (if \code{format = "df"}). In the case of a data.frame, the extracted elements will be joined
by \code{assessment_id}, which may result in an unexpected merged outcome when
multiple rows are present in one or more of the extracted elements (e.g.,
the \code{"threats"} element).}

\item{format}{(character) The format of the output. Either "list" or "df"
(for a data.frame).}

\item{flatten}{(logical) If \code{TRUE}, the output will be flattened to a
data.frame. Note that this may not work for all elements, especially
complex multilevel list elements. Only used when \code{format = "df"}. The
\code{dplyr}, \code{tidyr}, and \code{tibble} packages are required to use this feature.
Note that fields with no data across all assessments may be lost.}
}
\value{
A list or data.frame containing the extracted element from each
assessment.
}
\description{
Extract a given element from each of a list of assessments, such as the
output of \code{\link[=rl_assessment_list]{rl_assessment_list()}}. This is useful for extracting specific
details from the assessments, such as taxonomy (\code{el_name = "taxon"}),
synonyms (\code{el_name = "taxon__synonyms"}), habitats (\code{el_name = "habitats"}),
or the red list category (\code{el_name = "red_list_category"}).
}
\details{
The following top-level element names can be extracted as of
2025-08-29:
\itemize{
\item assessment_date
\item assessment_id
\item assessment_points
\item assessment_ranges
\item biogeographical_realms
\item citation
\item conservation_actions
\item credits
\item criteria
\item documentation
\item errata
\item faos
\item growth_forms
\item habitats
\item latest
\item lmes
\item locations
\item population_trend
\item possibly_extinct
\item possibly_extinct_in_the_wild
\item red_list_category
\item references
\item researches
\item scopes
\item sis_taxon_id
\item stresses
\item supplementary_info
\item systems
\item taxon
\item threats
\item url
\item use_and_trade
\item year_published
}

Note that there are also sublevels of the assessment hierarchy, but they
would be unwieldy to display here. A complete and up-to-date list can be
found by inspecting the return of an \code{\link[=rl_assessment]{rl_assessment()}} call.
}
\examples{
\dontrun{
lst <- rl_assessment_list(ids = c(166290968, 136250858))
# get complex elements as a list
ex1 <- rl_assessment_extract(lst, "taxon")
# get simple elements as a data.frame
ex2 <- rl_assessment_extract(lst, "red_list_category__code", format = "df")
# get complex elements as a data.frame
ex3 <- rl_assessment_extract(lst, "threats", format = "df")
# get the same elements flattened to a single data.frame
ex4 <- rl_assessment_extract(lst, "threats", format = "df", flatten = TRUE)
# get subelements flattened to a data.frame
ex5 <- rl_assessment_extract(lst, "taxon__order_name", format = "df",
                             flatten = TRUE)
# get a data frame with both taxon name and red list category code
ex6 <- rl_assessment_extract(lst, c("taxon", "red_list_category__code"),
                             format = "df", flatten = TRUE)
}
}
